@@ -33,7 +33,6 @@ use nautilus_model::{
     instruments::InstrumentAny,
     types::Currency,
 };
-use ustr::Ustr;
 
 use super::{
     decode::{
@@ -102,7 +101,7 @@ impl DatabentoDataLoader {
 
         loader
             .load_publishers(publishers_filepath)
-            .unwrap_or_else(|e| panic!("Error loading publishers.json: {e}",));
+            .unwrap_or_else(|e| panic!("Error loading publishers.json: {e}"));
 
         Ok(loader)
     }
@@ -116,17 +115,18 @@ impl DatabentoDataLoader {
             .clone()
             .into_iter()
             .map(|p| (p.publisher_id, p))
-            .collect::<IndexMap<u16, DatabentoPublisher>>();
+            .collect();
 
-        self.venue_dataset_map = publishers
-            .iter()
-            .map(|p| {
-                (
-                    Venue::from(p.venue.as_str()),
-                    Dataset::from(p.dataset.as_str()),
-                )
-            })
-            .collect::<IndexMap<Venue, Ustr>>();
+        let mut venue_dataset_map = IndexMap::new();
+
+        // Only insert a dataset if the venue key is not already in the map
+        for publisher in &publishers {
+            let venue = Venue::from(publisher.venue.as_str());
+            let dataset = Dataset::from(publisher.dataset.as_str());
+            venue_dataset_map.entry(venue).or_insert(dataset);
+        }
+
+        self.venue_dataset_map = venue_dataset_map;
 
         // Insert CME Globex exchanges
         let glbx = Dataset::from("GLBX.MDP3");
@@ -142,29 +142,35 @@ impl DatabentoDataLoader {
         self.publisher_venue_map = publishers
             .into_iter()
             .map(|p| (p.publisher_id, Venue::from(p.venue.as_str())))
-            .collect::<IndexMap<u16, Venue>>();
+            .collect();
 
         Ok(())
     }
 
-    /// Return the internal Databento publishers currently held by the loader.
+    /// Returns the internal Databento publishers currently held by the loader.
     #[must_use]
     pub const fn get_publishers(&self) -> &IndexMap<u16, DatabentoPublisher> {
         &self.publishers_map
     }
 
-    // Return the dataset which matches the given `venue` (if found).
+    /// Sets the `venue` to map to the given `dataset`.
+    pub fn set_dataset_for_venue(&mut self, dataset: Dataset, venue: Venue) {
+        _ = self.venue_dataset_map.insert(venue, dataset);
+    }
+
+    /// Returns the dataset which matches the given `venue` (if found).
     #[must_use]
     pub fn get_dataset_for_venue(&self, venue: &Venue) -> Option<&Dataset> {
         self.venue_dataset_map.get(venue)
     }
 
-    // Return the venue which matches the given `publisher_id` (if found).
+    /// Returns the venue which matches the given `publisher_id` (if found).
     #[must_use]
     pub fn get_venue_for_publisher(&self, publisher_id: PublisherId) -> Option<&Venue> {
         self.publisher_venue_map.get(&publisher_id)
     }
 
+    /// Returns the schema for the given `filepath`.
     pub fn schema_from_file(&self, filepath: &Path) -> anyhow::Result<Option<String>> {
         let decoder = Decoder::from_zstd_file(filepath)?;
         let metadata = decoder.metadata();
@@ -584,7 +590,8 @@ impl DatabentoDataLoader {
 mod tests {
     use std::path::{Path, PathBuf};
 
-    use rstest::*;
+    use rstest::{fixture, rstest};
+    use ustr::Ustr;
 
     use super::*;
 
@@ -592,27 +599,36 @@ mod tests {
         Path::new(env!("CARGO_MANIFEST_DIR")).join("test_data")
     }
 
-    fn data_loader() -> DatabentoDataLoader {
+    #[fixture]
+    fn loader() -> DatabentoDataLoader {
         let publishers_filepath = Path::new(env!("CARGO_MANIFEST_DIR")).join("publishers.json");
         DatabentoDataLoader::new(Some(publishers_filepath)).unwrap()
     }
 
     // TODO: Improve the below assertions that we've actually read the records we expected
 
+    #[rstest]
+    fn test_set_dataset_venue_mapping(mut loader: DatabentoDataLoader) {
+        let dataset = Ustr::from("EQUS.PLUS");
+        let venue = Venue::from("XNAS");
+        loader.set_dataset_for_venue(dataset, venue);
+
+        let result = loader.get_dataset_for_venue(&venue).unwrap();
+        assert_eq!(*result, dataset);
+    }
+
     #[rstest]
     // #[case(test_data_path().join("test_data.definition.dbn.zst"))] // TODO: Fails
     #[case(test_data_path().join("test_data.definition.v1.dbn.zst"))]
-    fn test_load_instruments(#[case] path: PathBuf) {
-        let mut loader = data_loader();
+    fn test_load_instruments(mut loader: DatabentoDataLoader, #[case] path: PathBuf) {
         let instruments = loader.load_instruments(&path, false).unwrap();
 
         assert_eq!(instruments.len(), 2);
     }
 
     #[rstest]
-    fn test_load_order_book_deltas() {
+    fn test_load_order_book_deltas(loader: DatabentoDataLoader) {
         let path = test_data_path().join("test_data.mbo.dbn.zst");
-        let loader = data_loader();
         let instrument_id = InstrumentId::from("ESM4.GLBX");
 
         let deltas = loader
@@ -623,9 +639,8 @@ mod tests {
     }
 
     #[rstest]
-    fn test_load_order_book_depth10() {
+    fn test_load_order_book_depth10(loader: DatabentoDataLoader) {
         let path = test_data_path().join("test_data.mbp-10.dbn.zst");
-        let loader = data_loader();
         let instrument_id = InstrumentId::from("ESM4.GLBX");
 
         let depths = loader
@@ -636,9 +651,8 @@ mod tests {
     }
 
     #[rstest]
-    fn test_load_quotes() {
+    fn test_load_quotes(loader: DatabentoDataLoader) {
         let path = test_data_path().join("test_data.mbp-1.dbn.zst");
-        let loader = data_loader();
         let instrument_id = InstrumentId::from("ESM4.GLBX");
 
         let quotes = loader
@@ -651,8 +665,7 @@ mod tests {
     #[rstest]
     #[case(test_data_path().join("test_data.bbo-1s.dbn.zst"))]
     #[case(test_data_path().join("test_data.bbo-1m.dbn.zst"))]
-    fn test_load_bbo_quotes(#[case] path: PathBuf) {
-        let loader = data_loader();
+    fn test_load_bbo_quotes(loader: DatabentoDataLoader, #[case] path: PathBuf) {
         let instrument_id = InstrumentId::from("ESM4.GLBX");
 
         let quotes = loader
@@ -663,9 +676,8 @@ mod tests {
     }
 
     #[rstest]
-    fn test_load_tbbo_trades() {
+    fn test_load_tbbo_trades(loader: DatabentoDataLoader) {
         let path = test_data_path().join("test_data.tbbo.dbn.zst");
-        let loader = data_loader();
         let instrument_id = InstrumentId::from("ESM4.GLBX");
 
         let _trades = loader
@@ -676,10 +688,8 @@ mod tests {
     }
 
     #[rstest]
-    fn test_load_trades() {
+    fn test_load_trades(loader: DatabentoDataLoader) {
         let path = test_data_path().join("test_data.trades.dbn.zst");
-        let loader = data_loader();
-
         let instrument_id = InstrumentId::from("ESM4.GLBX");
         let trades = loader
             .load_trades(&path, Some(instrument_id), None)
@@ -693,9 +703,7 @@ mod tests {
     #[case(test_data_path().join("test_data.ohlcv-1h.dbn.zst"))]
     #[case(test_data_path().join("test_data.ohlcv-1m.dbn.zst"))]
     #[case(test_data_path().join("test_data.ohlcv-1s.dbn.zst"))]
-    fn test_load_bars(#[case] path: PathBuf) {
-        let loader = data_loader();
-
+    fn test_load_bars(loader: DatabentoDataLoader, #[case] path: PathBuf) {
         let instrument_id = InstrumentId::from("ESM4.GLBX");
         let bars = loader.load_bars(&path, Some(instrument_id), None).unwrap();
 
