@@ -815,7 +815,7 @@ mod tests {
         identifiers::InstrumentId,
         types::Price,
     };
-    use nautilus_test_kit::common::{
+    use nautilus_testkit::common::{
         ensure_data_exists_tardis_binance_snapshot5, ensure_data_exists_tardis_binance_snapshot25,
         ensure_data_exists_tardis_bitmex_trades, ensure_data_exists_tardis_deribit_book_l2,
         ensure_data_exists_tardis_huobi_quotes,
@@ -824,6 +824,8 @@ mod tests {
 
     use super::*;
 
+    // TODO: Flakey in CI, potentially to do with syncing large test data files from cache
+    #[ignore = "Flakey test: called `Result::unwrap()` on an `Err` value: Error(Io(Kind(UnexpectedEof)))"]
     #[rstest]
     #[case(Some(1), Some(0))] // Explicit precisions
     #[case(None, None)] // Inferred precisions
@@ -856,6 +858,8 @@ mod tests {
         assert_eq!(deltas[0].ts_init, 1585699200355684000);
     }
 
+    // TODO: Flakey in CI, potentially to do with syncing large test data files from cache
+    #[ignore = "Flakey test: called `Result::unwrap()` on an `Err` value: Error(Io(Kind(UnexpectedEof)))"]
     #[rstest]
     #[case(Some(2), Some(3))] // Explicit precisions
     #[case(None, None)] // Inferred precisions
@@ -896,6 +900,8 @@ mod tests {
         assert_eq!(depths[0].sequence, 0);
     }
 
+    // TODO: Flakey in CI, potentially to do with syncing large test data files from cache
+    #[ignore = "Flakey test: called `Result::unwrap()` on an `Err` value: Error(Io(Kind(UnexpectedEof)))"]
     #[rstest]
     #[case(Some(2), Some(3))] // Explicit precisions
     #[case(None, None)] // Inferred precisions
@@ -936,6 +942,8 @@ mod tests {
         assert_eq!(depths[0].sequence, 0);
     }
 
+    // TODO: Flakey in CI, potentially to do with syncing large test data files from cache
+    #[ignore = "Flakey test: called `Result::unwrap()` on an `Err` value: Error(Io(Kind(UnexpectedEof)))"]
     #[rstest]
     #[case(Some(1), Some(0))] // Explicit precisions
     #[case(None, None)] // Inferred precisions
@@ -954,7 +962,10 @@ mod tests {
         .unwrap();
 
         assert_eq!(quotes.len(), 10_000);
-        assert_eq!(quotes[0].instrument_id, InstrumentId::from("BTC-USD.HUOBI"));
+        assert_eq!(
+            quotes[0].instrument_id,
+            InstrumentId::from("BTC-USD.HUOBI_DELIVERY")
+        );
         assert_eq!(quotes[0].bid_price, Price::from("8629.2"));
         assert_eq!(quotes[0].bid_size, Quantity::from("806"));
         assert_eq!(quotes[0].ask_price, Price::from("8629.3"));
@@ -963,6 +974,8 @@ mod tests {
         assert_eq!(quotes[0].ts_init, 1588291201234268000);
     }
 
+    // TODO: Flakey in CI, potentially to do with syncing large test data files from cache
+    #[ignore = "Flakey test: called `Result::unwrap()` on an `Err` value: Error(Io(Kind(UnexpectedEof)))"]
     #[rstest]
     #[case(Some(1), Some(0))] // Explicit precisions
     #[case(None, None)] // Inferred precisions
