@@ -25,9 +25,9 @@ use std::{
     rc::Rc,
 };
 
-use nautilus_common::{cache::Cache, msgbus::MessageBus};
+use nautilus_common::{cache::Cache, clock::Clock};
 use nautilus_core::{
-    AtomicTime, UnixNanos,
+    UnixNanos,
     correctness::{FAILED, check_equal},
 };
 use nautilus_execution::{
@@ -105,8 +105,7 @@ pub struct SimulatedExchange {
     matching_engines: HashMap<InstrumentId, OrderMatchingEngine>,
     leverages: HashMap<InstrumentId, Decimal>,
     modules: Vec<Box<dyn SimulationModule>>,
-    clock: &'static AtomicTime,
-    msgbus: Rc<RefCell<MessageBus>>,
+    clock: Rc<RefCell<dyn Clock>>,
     cache: Rc<RefCell<Cache>>,
     message_queue: VecDeque<TradingCommand>,
     inflight_queue: BinaryHeap<InflightCommand>,
@@ -134,9 +133,8 @@ impl SimulatedExchange {
         default_leverage: Decimal,
         leverages: HashMap<InstrumentId, Decimal>,
         modules: Vec<Box<dyn SimulationModule>>,
-        msgbus: Rc<RefCell<MessageBus>>, // TODO add portfolio
         cache: Rc<RefCell<Cache>>,
-        clock: &'static AtomicTime,
+        clock: Rc<RefCell<dyn Clock>>,
         fill_model: FillModel,
         fee_model: FeeModelAny,
         book_type: BookType,
@@ -175,7 +173,6 @@ impl SimulatedExchange {
             leverages,
             modules,
             clock,
-            msgbus,
             cache,
             message_queue: VecDeque::new(),
             inflight_queue: BinaryHeap::new(),
@@ -252,8 +249,7 @@ impl SimulatedExchange {
             self.book_type,
             self.oms_type,
             self.account_type,
-            self.clock,
-            Rc::clone(&self.msgbus),
+            self.clock.clone(),
             Rc::clone(&self.cache),
             matching_engine_config,
         );
@@ -284,8 +280,11 @@ impl SimulatedExchange {
     }
 
     #[must_use]
-    pub fn get_matching_engine(&self, instrument_id: InstrumentId) -> Option<&OrderMatchingEngine> {
-        self.matching_engines.get(&instrument_id)
+    pub fn get_matching_engine(
+        &self,
+        instrument_id: &InstrumentId,
+    ) -> Option<&OrderMatchingEngine> {
+        self.matching_engines.get(instrument_id)
     }
 
     #[must_use]
@@ -384,7 +383,7 @@ impl SimulatedExchange {
                                     vec![current_balance],
                                     margins.values().copied().collect(),
                                     true,
-                                    self.clock.get_time_ns(),
+                                    self.clock.borrow().timestamp_ns(),
                                 )
                                 .unwrap();
                         }
@@ -610,7 +609,7 @@ impl SimulatedExchange {
     }
 
     pub fn process(&mut self, ts_now: UnixNanos) {
-        self.clock.set_time(ts_now);
+        // TODO implement correct clock fixed time setting self.clock.set_time(ts_now);
 
         // Process inflight commands
         while let Some(inflight) = self.inflight_queue.peek() {
@@ -688,7 +687,7 @@ impl SimulatedExchange {
 
         if let Some(exec_client) = &self.exec_client {
             exec_client
-                .generate_account_state(balances, vec![], true, self.clock.get_time_ns())
+                .generate_account_state(balances, vec![], true, self.clock.borrow().timestamp_ns())
                 .unwrap();
         }
 
@@ -720,7 +719,7 @@ mod tests {
         cache::Cache,
         clock::TestClock,
         msgbus::{
-            MessageBus, register,
+            self,
             stubs::{get_message_saving_handler, get_saved_messages},
         },
     };
@@ -767,12 +766,10 @@ mod tests {
         venue: Venue,
         account_type: AccountType,
         book_type: BookType,
-        msgbus: Option<Rc<RefCell<MessageBus>>>,
         cache: Option<Rc<RefCell<Cache>>>,
     ) -> Rc<RefCell<SimulatedExchange>> {
-        let msgbus = msgbus.unwrap_or(Rc::new(RefCell::new(MessageBus::default())));
         let cache = cache.unwrap_or(Rc::new(RefCell::new(Cache::default())));
-
+        let clock = Rc::new(RefCell::new(TestClock::new()));
         let exchange = Rc::new(RefCell::new(
             SimulatedExchange::new(
                 venue,
@@ -783,9 +780,8 @@ mod tests {
                 1.into(),
                 HashMap::new(),
                 vec![],
-                msgbus.clone(),
                 cache.clone(),
-                &ATOMIC_TIME,
+                clock,
                 FillModel::default(),
                 FeeModelAny::MakerTaker(MakerTakerFeeModel),
                 book_type,
@@ -809,7 +805,6 @@ mod tests {
             AccountId::default(),
             exchange.clone(),
             cache.clone(),
-            msgbus.clone(),
             Rc::new(RefCell::new(clock)),
             None,
             None,
@@ -857,7 +852,6 @@ mod tests {
             AccountType::Margin,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
         exchange.borrow_mut().add_instrument(instrument).unwrap();
@@ -871,7 +865,6 @@ mod tests {
             AccountType::Cash,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
         exchange.borrow_mut().add_instrument(instrument).unwrap();
@@ -884,7 +877,6 @@ mod tests {
             AccountType::Margin,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -920,7 +912,6 @@ mod tests {
             AccountType::Margin,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -956,7 +947,6 @@ mod tests {
             AccountType::Margin,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -994,7 +984,6 @@ mod tests {
             AccountType::Margin,
             BookType::L1_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -1046,7 +1035,6 @@ mod tests {
             AccountType::Margin,
             BookType::L2_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -1107,7 +1095,6 @@ mod tests {
             AccountType::Margin,
             BookType::L2_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -1180,7 +1167,6 @@ mod tests {
             AccountType::Margin,
             BookType::L2_MBP,
             None,
-            None,
         );
         let instrument = InstrumentAny::CryptoPerpetual(crypto_perpetual_ethusdt);
 
@@ -1205,7 +1191,7 @@ mod tests {
 
         let market_status = exchange
             .borrow()
-            .get_matching_engine(crypto_perpetual_ethusdt.id)
+            .get_matching_engine(&crypto_perpetual_ethusdt.id)
             .unwrap()
             .market_status;
         assert_eq!(market_status, MarketStatus::Closed);
@@ -1214,10 +1200,9 @@ mod tests {
     #[rstest]
     fn test_accounting() {
         let account_type = AccountType::Margin;
-        let msgbus = MessageBus::default().register_message_bus();
         let mut cache = Cache::default();
         let handler = get_message_saving_handler::<AccountState>(None);
-        register(Ustr::from("Portfolio.update_account"), handler.clone());
+        msgbus::register(Ustr::from("Portfolio.update_account"), handler.clone());
         let margin_account = MarginAccount::new(
             AccountState::new(
                 AccountId::from("SIM-001"),
@@ -1246,7 +1231,6 @@ mod tests {
             Venue::new("SIM"),
             account_type,
             BookType::L2_MBP,
-            Some(msgbus.clone()),
             Some(Rc::new(RefCell::new(cache))),
         );
         exchange.borrow_mut().initialize_account();
@@ -1314,12 +1298,10 @@ mod tests {
 
     #[rstest]
     fn test_process_without_latency_model(crypto_perpetual_ethusdt: CryptoPerpetual) {
-        let msgbus = MessageBus::default().register_message_bus();
         let exchange = get_exchange(
             Venue::new("BINANCE"),
             AccountType::Margin,
             BookType::L2_MBP,
-            Some(msgbus.clone()),
             None,
         );
 
@@ -1351,12 +1333,10 @@ mod tests {
             UnixNanos::from(300),
             UnixNanos::from(100),
         );
-        let msgbus = MessageBus::default().register_message_bus();
         let exchange = get_exchange(
             Venue::new("BINANCE"),
             AccountType::Margin,
             BookType::L2_MBP,
-            Some(msgbus.clone()),
             None,
         );
         exchange.borrow_mut().set_latency_model(latency_model);
