@@ -292,11 +292,12 @@ impl Log for Logger {
             } else {
                 get_atomic_clock_static().get_time_ns()
             };
+            let level = record.level();
             let key_values = record.key_values();
-            let color = key_values
+            let color: LogColor = key_values
                 .get("color".into())
                 .and_then(|v| v.to_u64().map(|v| (v as u8).into()))
-                .unwrap_or(LogColor::Normal);
+                .unwrap_or(level.into());
             let component = key_values.get("component".into()).map_or_else(
                 || Ustr::from(record.metadata().target()),
                 |v| Ustr::from(&v.to_string()),
@@ -304,7 +305,7 @@ impl Log for Logger {
 
             let line = LogLine {
                 timestamp,
-                level: record.level(),
+                level,
                 color,
                 component,
                 message: format!("{}", record.args()),
@@ -531,7 +532,7 @@ impl Drop for LogGuard {
 ////////////////////////////////////////////////////////////////////////////////
 #[cfg(test)]
 mod tests {
-    use std::{collections::HashMap, time::Duration};
+    use std::{collections::HashMap, thread::sleep, time::Duration};
 
     use log::LevelFilter;
     use nautilus_core::UUID4;
@@ -774,4 +775,94 @@ mod tests {
             "{\"timestamp\":\"1970-01-20T02:20:00.000000000Z\",\"trader_id\":\"TRADER-001\",\"level\":\"INFO\",\"color\":\"NORMAL\",\"component\":\"RiskEngine\",\"message\":\"This is a test.\"}\n"
         );
     }
+
+    #[ignore = "Flaky test: Passing locally on some systems, failing in CI"]
+    #[rstest]
+    fn test_file_rotation_and_backup_limits() {
+        // Create a temporary directory for log files
+        let temp_dir = tempdir().expect("Failed to create temporary directory");
+        let dir_path = temp_dir.path().to_str().unwrap().to_string();
+
+        // Configure a small max file size to trigger rotation quickly
+        let max_backups = 3;
+        let max_file_size = 100;
+        let file_config = FileWriterConfig {
+            directory: Some(dir_path.clone()),
+            file_name: None,
+            file_format: Some("log".to_string()),
+            file_rotate: Some((max_file_size, max_backups).into()), // 100 bytes max size, 3 max backups
+        };
+
+        // Create the file writer
+        let config = LoggerConfig::from_spec("fileout=Info;Test=Info").unwrap();
+        let log_guard = Logger::init_with_config(
+            TraderId::from("TRADER-001"),
+            UUID4::new(),
+            config,
+            file_config,
+        );
+
+        log::info!(
+            component = "Test";
+            "Test log message with enough content to exceed our small max file size limit"
+        );
+
+        sleep(Duration::from_millis(100));
+
+        // Count the number of log files in the directory
+        let files: Vec<_> = std::fs::read_dir(&dir_path)
+            .expect("Failed to read directory")
+            .filter_map(Result::ok)
+            .filter(|entry| entry.path().extension().map_or(false, |ext| ext == "log"))
+            .collect();
+
+        // We should have multiple files due to rotation
+        assert_eq!(files.len(), 1);
+
+        log::info!(
+            component = "Test";
+            "Test log message with enough content to exceed our small max file size limit"
+        );
+
+        sleep(Duration::from_millis(100));
+
+        // Count the number of log files in the directory
+        let files: Vec<_> = std::fs::read_dir(&dir_path)
+            .expect("Failed to read directory")
+            .filter_map(Result::ok)
+            .filter(|entry| entry.path().extension().map_or(false, |ext| ext == "log"))
+            .collect();
+
+        // We should have multiple files due to rotation
+        assert_eq!(files.len(), 2);
+
+        for _ in 0..5 {
+            // Write enough data to trigger a few rotations
+            log::info!(
+            component = "Test";
+            "Test log message with enough content to exceed our small max file size limit"
+            );
+
+            sleep(Duration::from_millis(100));
+        }
+
+        // Count the number of log files in the directory
+        let files: Vec<_> = std::fs::read_dir(&dir_path)
+            .expect("Failed to read directory")
+            .filter_map(Result::ok)
+            .filter(|entry| entry.path().extension().map_or(false, |ext| ext == "log"))
+            .collect();
+
+        // We should have at most max_backups + 1 files (current file + backups)
+        assert!(
+            files.len() == max_backups as usize + 1,
+            "Expected at most {} log files, found {}",
+            max_backups,
+            files.len()
+        );
+
+        // Clean up
+        drop(log_guard);
+        drop(temp_dir);
+    }
 }
