@@ -36,7 +36,8 @@ use nautilus_common::{
     generators::position_id::PositionIdGenerator,
     logging::{CMD, EVT, RECV},
     msgbus::{
-        MessageBus, {self},
+        self, get_message_bus,
+        switchboard::{self},
     },
 };
 use nautilus_core::UUID4;
@@ -65,7 +66,6 @@ use crate::{
 pub struct ExecutionEngine {
     clock: Rc<RefCell<dyn Clock>>,
     cache: Rc<RefCell<Cache>>,
-    msgbus: Rc<RefCell<MessageBus>>,
     clients: HashMap<ClientId, Rc<dyn ExecutionClient>>,
     default_client: Option<Rc<dyn ExecutionClient>>,
     routing_map: HashMap<Venue, ClientId>,
@@ -79,14 +79,12 @@ impl ExecutionEngine {
     pub fn new(
         clock: Rc<RefCell<dyn Clock>>,
         cache: Rc<RefCell<Cache>>,
-        msgbus: Rc<RefCell<MessageBus>>,
         config: Option<ExecutionEngineConfig>,
     ) -> Self {
-        let trader_id = msgbus.borrow().trader_id;
+        let trader_id = get_message_bus().borrow().trader_id;
         Self {
             clock: clock.clone(),
             cache,
-            msgbus,
             clients: HashMap::new(),
             default_client: None,
             routing_map: HashMap::new(),
@@ -127,7 +125,7 @@ impl ExecutionEngine {
         self.external_order_claims.keys().copied().collect()
     }
 
-    // -- REGISTRATION --------------------------------------------------------
+    // -- REGISTRATION ----------------------------------------------------------------------------
 
     pub fn register_client(&mut self, client: Rc<dyn ExecutionClient>) -> anyhow::Result<()> {
         if self.clients.contains_key(&client.client_id()) {
@@ -147,6 +145,11 @@ impl ExecutionEngine {
         self.default_client = Some(client);
     }
 
+    #[must_use]
+    pub fn get_client(&self, client_id: &ClientId) -> Option<Rc<dyn ExecutionClient>> {
+        self.clients.get(client_id).cloned()
+    }
+
     pub fn register_venue_routing(
         &mut self,
         client_id: ClientId,
@@ -178,7 +181,7 @@ impl ExecutionEngine {
         }
     }
 
-    // -- COMMANDS ------------------------------------------------------------
+    // -- COMMANDS --------------------------------------------------------------------------------
 
     #[allow(clippy::await_holding_refcell_ref)]
     pub async fn load_cache(&mut self) -> anyhow::Result<()> {
@@ -228,7 +231,7 @@ impl ExecutionEngine {
         self.execute_command(command);
     }
 
-    // -- COMMAND HANDLERS ----------------------------------------------------
+    // -- COMMAND HANDLERS ------------------------------------------------------------------------
 
     fn execute_command(&self, command: TradingCommand) {
         if self.config.debug {
@@ -475,11 +478,8 @@ impl ExecutionEngine {
             }
         }
 
-        let mut msgbus = self.msgbus.borrow_mut();
-        if msgbus.has_backing {
-            let topic = msgbus
-                .switchboard
-                .get_order_snapshots_topic(order.client_order_id());
+        if get_message_bus().borrow().has_backing {
+            let topic = switchboard::get_order_snapshots_topic(order.client_order_id());
             msgbus::publish(&topic, order);
         }
     }
@@ -494,14 +494,11 @@ impl ExecutionEngine {
         //     position.unrealized_pnl(last)
         // }
 
-        let mut msgbus = self.msgbus.borrow_mut();
-        let topic = msgbus
-            .switchboard
-            .get_positions_snapshots_topic(position.id);
+        let topic = switchboard::get_positions_snapshots_topic(position.id);
         msgbus::publish(&topic, position);
     }
 
-    // -- EVENT HANDLERS ----------------------------------------------------
+    // -- EVENT HANDLERS --------------------------------------------------------------------------
 
     fn handle_event(&mut self, event: &OrderEventAny) {
         if self.config.debug {
@@ -606,7 +603,7 @@ impl ExecutionEngine {
         // Check if position ID already exists
         if let Some(position_id) = fill.position_id {
             if self.config.debug {
-                log::debug!("Already had a position ID of: {}", position_id);
+                log::debug!("Already had a position ID of: {position_id}");
             }
             return position_id;
         }
@@ -665,10 +662,7 @@ impl ExecutionEngine {
             log::error!("Error updating order in cache: {e}");
         }
 
-        let mut msgbus = self.msgbus.borrow_mut();
-        let topic = msgbus
-            .switchboard
-            .get_event_orders_topic(event.strategy_id());
+        let topic = switchboard::get_event_orders_topic(event.strategy_id());
         msgbus::publish(&topic, order);
 
         if self.config.snapshot_orders {
@@ -699,7 +693,7 @@ impl ExecutionEngine {
         let position_id = if let Some(position_id) = fill.position_id {
             position_id
         } else {
-            log::error!("Cannot handle order fill: no position ID found for fill {fill}",);
+            log::error!("Cannot handle order fill: no position ID found for fill {fill}");
             return;
         };
 
@@ -765,10 +759,7 @@ impl ExecutionEngine {
 
         let ts_init = self.clock.borrow().timestamp_ns();
         let event = PositionOpened::create(&position, &fill, UUID4::new(), ts_init);
-        let mut msgbus = self.msgbus.borrow_mut();
-        let topic = msgbus
-            .switchboard
-            .get_event_positions_topic(event.strategy_id);
+        let topic = switchboard::get_event_positions_topic(event.strategy_id);
         msgbus::publish(&topic, &event);
 
         Ok(position)
@@ -786,10 +777,7 @@ impl ExecutionEngine {
             self.create_position_state_snapshot(position);
         }
 
-        let mut msgbus = self.msgbus.borrow_mut();
-        let topic = msgbus
-            .switchboard
-            .get_event_positions_topic(position.strategy_id);
+        let topic = switchboard::get_event_positions_topic(position.strategy_id);
         let ts_init = self.clock.borrow().timestamp_ns();
 
         if position.is_closed() {
@@ -923,7 +911,7 @@ impl ExecutionEngine {
         }
     }
 
-    // -- INTERNAL ------------------------------------------------------------
+    // -- INTERNAL --------------------------------------------------------------------------------
 
     fn set_position_id_counts(&mut self) {
         // For the internal position ID generator
@@ -1056,10 +1044,7 @@ impl ExecutionEngine {
             return;
         }
 
-        let mut msgbus = self.msgbus.borrow_mut();
-        let topic = msgbus
-            .switchboard
-            .get_event_orders_topic(order.strategy_id());
+        let topic = switchboard::get_event_orders_topic(order.strategy_id());
         msgbus::publish(&topic, &denied);
 
         if self.config.snapshot_orders {
@@ -1107,12 +1092,11 @@ mod tests {
 
     // Helpers
     fn _get_exec_engine(
-        msgbus: Rc<RefCell<MessageBus>>,
         cache: Rc<RefCell<Cache>>,
         clock: Rc<RefCell<TestClock>>,
         config: Option<ExecutionEngineConfig>,
     ) -> ExecutionEngine {
-        ExecutionEngine::new(clock, cache, msgbus, config)
+        ExecutionEngine::new(clock, cache, config)
     }
 
     // TODO: After Implementing ExecutionClient & Strategy
