@@ -163,7 +163,7 @@ impl RedisCacheDatabase {
         let encoding = config.encoding;
         let handle = get_runtime().spawn(async move {
             if let Err(e) = process_commands(rx, trader_key_clone, config.clone()).await {
-                log::error!("Failed to spawn task '{CACHE_WRITE}': {e}");
+                log::error!("Error in task '{CACHE_WRITE}': {e}");
             }
         });
 
@@ -189,7 +189,7 @@ impl RedisCacheDatabase {
         log::debug!("Closing");
 
         if let Err(e) = self.tx.send(DatabaseCommand::close()) {
-            log::debug!("Error sending close message: {e:?}")
+            log::debug!("Error sending close command: {e:?}")
         }
 
         log::debug!("Awaiting task '{CACHE_WRITE}'");
@@ -271,13 +271,17 @@ async fn process_commands(
             last_drain = Instant::now();
         } else {
             match rx.recv().await {
-                Some(msg) => {
-                    if let DatabaseOperation::Close = msg.op_type {
+                Some(cmd) => {
+                    tracing::debug!("Received {cmd:?}");
+                    if let DatabaseOperation::Close = cmd.op_type {
                         break;
                     }
-                    buffer.push_back(msg)
+                    buffer.push_back(cmd)
+                }
+                None => {
+                    tracing::debug!("Command channel closed");
+                    break;
                 }
-                None => break, // Channel hung up
             }
         }
     }
@@ -622,13 +626,24 @@ impl CacheDatabaseAdapter for RedisCacheDatabaseAdapter {
     async fn load_all(&self) -> anyhow::Result<CacheMap> {
         tracing::debug!("Loading all data");
 
-        let (currencies, instruments, synthetics, accounts, orders, positions) = try_join!(
+        let (
+            currencies,
+            instruments,
+            synthetics,
+            accounts,
+            orders,
+            positions,
+            greeks,
+            yield_curves,
+        ) = try_join!(
             self.load_currencies(),
             self.load_instruments(),
             self.load_synthetics(),
             self.load_accounts(),
             self.load_orders(),
-            self.load_positions()
+            self.load_positions(),
+            self.load_greeks(),
+            self.load_yield_curves()
         )
         .map_err(|e| anyhow::anyhow!("Error loading cache data: {e}"))?;
 
@@ -639,6 +654,8 @@ impl CacheDatabaseAdapter for RedisCacheDatabaseAdapter {
             accounts,
             orders,
             positions,
+            greeks,
+            yield_curves,
         })
     }
 
