@@ -121,11 +121,17 @@ impl PostgresCacheDatabase {
                 last_drain = Instant::now();
             } else {
                 match rx.recv().await {
-                    Some(msg) => match msg {
-                        DatabaseQuery::Close => break,
-                        _ => buffer.push_back(msg),
-                    },
-                    None => break, // Channel hung up
+                    Some(msg) => {
+                        tracing::debug!("Received {msg:?}");
+                        match msg {
+                            DatabaseQuery::Close => break,
+                            _ => buffer.push_back(msg),
+                        }
+                    }
+                    None => {
+                        tracing::debug!("Command channel closed");
+                        break;
+                    }
                 }
             }
         }
@@ -215,6 +221,11 @@ impl CacheDatabaseAdapter for PostgresCacheDatabase {
         )
         .map_err(|e| anyhow::anyhow!("Error loading cache data: {}", e))?;
 
+        // For now, we don't load greeks and yield curves from the database
+        // This will be implemented in the future
+        let greeks = HashMap::new();
+        let yield_curves = HashMap::new();
+
         Ok(CacheMap {
             currencies,
             instruments,
@@ -222,6 +233,8 @@ impl CacheDatabaseAdapter for PostgresCacheDatabase {
             accounts,
             orders,
             positions,
+            greeks,
+            yield_curves,
         })
     }
 
