@@ -110,7 +110,7 @@ impl<'r> FromRow<'r, PgRow> for OrderEventAnyModel {
             let model = OrderUpdatedModel::from_row(row)?;
             Ok(OrderEventAnyModel(OrderEventAny::Updated(model.0)))
         } else {
-            panic!("Unknown order event kind: {kind} in Postgres transformation",)
+            panic!("Unknown order event kind: {kind} in Postgres transformation")
         }
     }
 }
