@@ -17,11 +17,13 @@
 
 pub mod bar;
 pub mod bet;
+pub mod close;
 pub mod delta;
 pub mod deltas;
 pub mod depth;
 pub mod greeks;
 pub mod order;
+pub mod prices;
 pub mod quote;
 pub mod status;
 pub mod trade;
@@ -32,10 +34,10 @@ pub mod stubs;
 use std::{
     fmt::{Debug, Display},
     hash::{Hash, Hasher},
-    num::NonZeroU64,
     str::FromStr,
 };
 
+use close::InstrumentClose;
 use indexmap::IndexMap;
 use nautilus_core::UnixNanos;
 use serde::{Deserialize, Serialize};
@@ -47,16 +49,17 @@ pub use bar::{Bar, BarSpecification, BarType};
 pub use delta::OrderBookDelta;
 pub use deltas::{OrderBookDeltas, OrderBookDeltas_API};
 pub use depth::{DEPTH10_LEN, OrderBookDepth10};
-pub use greeks::{BlackScholesGreeksResult, black_scholes_greeks};
+pub use greeks::{
+    BlackScholesGreeksResult, GreeksData, PortfolioGreeks, YieldCurveData, black_scholes_greeks,
+    imply_vol_and_greeks,
+};
 pub use order::{BookOrder, NULL_ORDER};
+pub use prices::{IndexPriceUpdate, MarkPriceUpdate};
 pub use quote::QuoteTick;
 pub use status::InstrumentStatus;
 pub use trade::TradeTick;
 
-use crate::{
-    enums::BookType,
-    identifiers::{InstrumentId, Venue},
-};
+use crate::identifiers::{InstrumentId, Venue};
 
 /// A built-in Nautilus data type.
 ///
@@ -71,6 +74,9 @@ pub enum Data {
     Quote(QuoteTick),
     Trade(TradeTick),
     Bar(Bar),
+    MarkPriceUpdate(MarkPriceUpdate), // TODO: Rename to MarkPrice once Cython gone
+    IndexPriceUpdate(IndexPriceUpdate), // TODO: Rename to IndexPrice once Cython gone
+    InstrumentClose(InstrumentClose),
 }
 
 macro_rules! impl_try_from_data {
@@ -104,6 +110,9 @@ impl_try_from_data!(Delta, OrderBookDelta);
 impl_try_from_data!(Deltas, OrderBookDeltas_API);
 impl_try_from_data!(Trade, TradeTick);
 impl_try_from_data!(Bar, Bar);
+impl_try_from_data!(MarkPriceUpdate, MarkPriceUpdate);
+impl_try_from_data!(IndexPriceUpdate, IndexPriceUpdate);
+impl_try_from_data!(InstrumentClose, InstrumentClose);
 
 pub fn to_variant<T: TryFrom<Data>>(data: Vec<Data>) -> Vec<T> {
     data.into_iter()
@@ -121,6 +130,9 @@ impl Data {
             Self::Quote(quote) => quote.instrument_id,
             Self::Trade(trade) => trade.instrument_id,
             Self::Bar(bar) => bar.bar_type.instrument_id(),
+            Self::MarkPriceUpdate(mark_price) => mark_price.instrument_id,
+            Self::IndexPriceUpdate(index_price) => index_price.instrument_id,
+            Self::InstrumentClose(close) => close.instrument_id,
         }
     }
 
@@ -143,6 +155,9 @@ impl GetTsInit for Data {
             Self::Quote(q) => q.ts_init,
             Self::Trade(t) => t.ts_init,
             Self::Bar(b) => b.ts_init,
+            Self::MarkPriceUpdate(p) => p.ts_init,
+            Self::IndexPriceUpdate(p) => p.ts_init,
+            Self::InstrumentClose(c) => c.ts_init,
         }
     }
 }
@@ -188,6 +203,24 @@ impl From<Bar> for Data {
     }
 }
 
+impl From<MarkPriceUpdate> for Data {
+    fn from(value: MarkPriceUpdate) -> Self {
+        Self::MarkPriceUpdate(value)
+    }
+}
+
+impl From<IndexPriceUpdate> for Data {
+    fn from(value: IndexPriceUpdate) -> Self {
+        Self::IndexPriceUpdate(value)
+    }
+}
+
+impl From<InstrumentClose> for Data {
+    fn from(value: InstrumentClose) -> Self {
+        Self::InstrumentClose(value)
+    }
+}
+
 // TODO: https://blog.rust-lang.org/2024/03/30/i128-layout-update.html
 // i128 and u128 is now FFI compatible. However, since the clippy lint
 // hasn't been removed yet. We'll suppress with #[cfg_attr(feature = "high-precision", allow(improper_ctypes_definitions))]
@@ -290,22 +323,6 @@ impl DataType {
         Some(Venue::from(venue_str.as_str()))
     }
 
-    /// Returns a [`BarType`] from the metadata.
-    ///
-    /// # Panics
-    ///
-    /// This function panics:
-    /// - If there is no metadata.
-    /// - If the metadata does not contain a `bar_type`.
-    /// - If the `bar_type` value contained in the metadata is invalid.
-    pub fn bar_type(&self) -> BarType {
-        let metadata = self.metadata.as_ref().expect("metadata was `None`");
-        let bar_type_str = metadata
-            .get("bar_type")
-            .expect("No 'bar_type' found in metadata");
-        BarType::from_str(bar_type_str).expect("Invalid `BarType` for 'bar_type'")
-    }
-
     /// Returns an [`Option<UnixNanos>`] start from the metadata.
     ///
     /// # Panics
@@ -348,71 +365,6 @@ impl DataType {
                 .expect("Invalid `usize` for 'limit'"),
         )
     }
-
-    /// Returns a [`BookType`] from the metadata.
-    ///
-    /// # Panics
-    ///
-    /// This function panics:
-    /// - If there is no metadata.
-    /// - If the `book_type` value contained in the metadata is invalid.
-    pub fn book_type(&self) -> BookType {
-        let metadata = self.metadata.as_ref().expect("metadata was `None`");
-        let book_type_str = metadata
-            .get("book_type")
-            .expect("'book_type' not found in metadata");
-        BookType::from_str(book_type_str).expect("Invalid `BookType` for 'book_type'")
-    }
-
-    /// Returns an [`Option<usize>`] depth from the metadata.
-    ///
-    /// # Panics
-    ///
-    /// This function panics:
-    /// - If there is no metadata.
-    /// - If the `depth` value contained in the metadata is invalid.
-    pub fn depth(&self) -> Option<usize> {
-        let metadata = self.metadata.as_ref()?;
-        let depth_str = metadata.get("depth")?;
-        Some(
-            depth_str
-                .parse::<usize>()
-                .expect("Invalid `usize` for 'depth'"),
-        )
-    }
-
-    /// Returns a [`NonZeroU64`] interval (milliseconds) from the metadata.
-    ///
-    /// # Panics
-    ///
-    /// This function panics:
-    /// - If there is no metadata.
-    /// - If the `interval_ms` value contained in the metadata is invalid.
-    pub fn interval_ms(&self) -> NonZeroU64 {
-        let metadata = self.metadata.as_ref().expect("metadata was `None`");
-        let interval_ms_str = metadata
-            .get("interval_ms")
-            .expect("No 'interval_ms' in metadata");
-
-        interval_ms_str
-            .parse::<NonZeroU64>()
-            .expect("Invalid `NonZeroU64` for 'interval_ms'")
-    }
-
-    /// Returns a [`bool`] from the metadata indicating whether the book should be managed.
-    ///
-    /// # Panics
-    ///
-    /// This function panics:
-    /// - If there is no metadata.
-    /// - If the `managed` value contained in the metadata is invalid.
-    pub fn managed(&self) -> bool {
-        let metadata = self.metadata.as_ref().expect("metadata was `None`");
-        let managed_str = metadata.get("managed").expect("No 'managed' in metadata");
-        managed_str
-            .parse::<bool>()
-            .expect("Invalid `bool` for 'managed'")
-    }
 }
 
 impl PartialEq for DataType {
@@ -581,7 +533,7 @@ mod tests {
         assert_eq!(hash1, hash2);
     }
 
-    #[test]
+    #[rstest]
     fn test_data_type_display() {
         let metadata = Some(
             [("key1".to_string(), "value1".to_string())]
@@ -594,7 +546,7 @@ mod tests {
         assert_eq!(format!("{}", data_type), "ExampleType.key1=value1");
     }
 
-    #[test]
+    #[rstest]
     fn test_data_type_debug() {
         let metadata = Some(
             [("key1".to_string(), "value1".to_string())]
@@ -641,23 +593,6 @@ mod tests {
         assert_eq!(data_type.venue().unwrap(), Venue::new(venue_str));
     }
 
-    #[rstest]
-    fn test_parse_bar_type_from_metadata() {
-        let bar_type_str = "MSFT.XNAS-1000-TICK-LAST-INTERNAL";
-        let metadata = Some(
-            [("bar_type".to_string(), bar_type_str.to_string())]
-                .iter()
-                .cloned()
-                .collect(),
-        );
-        let data_type = DataType::new(stringify!(BarType), metadata);
-
-        assert_eq!(
-            data_type.bar_type(),
-            BarType::from_str(bar_type_str).unwrap()
-        );
-    }
-
     #[rstest]
     fn test_parse_start_from_metadata() {
         let start_ns = 1600054595844758000;
@@ -699,35 +634,4 @@ mod tests {
 
         assert_eq!(data_type.limit().unwrap(), limit);
     }
-
-    #[rstest]
-    fn test_parse_book_type_from_metadata() {
-        let book_type_str = "L3_MBO";
-        let metadata = Some(
-            [("book_type".to_string(), book_type_str.to_string())]
-                .iter()
-                .cloned()
-                .collect(),
-        );
-        let data_type = DataType::new(stringify!(OrderBookDelta), metadata);
-
-        assert_eq!(
-            data_type.book_type(),
-            BookType::from_str(book_type_str).unwrap()
-        );
-    }
-
-    #[rstest]
-    fn test_parse_depth_from_metadata() {
-        let depth = 25;
-        let metadata = Some(
-            [("depth".to_string(), depth.to_string())]
-                .iter()
-                .cloned()
-                .collect(),
-        );
-        let data_type = DataType::new(stringify!(OrderBookDeltas), metadata);
-
-        assert_eq!(data_type.depth().unwrap(), depth);
-    }
 }
