@@ -580,6 +580,17 @@ pub enum InstrumentCloseType {
     ContractExpired = 2,
 }
 
+/// Convert the given `value` to an [`InstrumentCloseType`].
+impl FromU8 for InstrumentCloseType {
+    fn from_u8(value: u8) -> Option<Self> {
+        match value {
+            1 => Some(Self::EndOfSession),
+            2 => Some(Self::ContractExpired),
+            _ => None,
+        }
+    }
+}
+
 /// The liqudity side for a trade.
 #[repr(C)]
 #[derive(
@@ -1169,19 +1180,19 @@ impl RecordFlag {
     pyo3::pyclass(eq, eq_int, module = "nautilus_trader.core.nautilus_pyo3.model.enums")
 )]
 pub enum TimeInForce {
-    /// Good-Till-Canceled (GTC) - the order remains active until canceled.
+    /// Good Till Cancel (GTC) - Remains active until canceled.
     Gtc = 1,
-    /// Immediate-Or-Cancel (IOC) - the order is filled as much as possible, the rest is canceled.
+    /// Immediate or Cancel (IOC) - Executes immediately to the extent possible, with any unfilled portion canceled.
     Ioc = 2,
-    /// Fill-Or-Kill (FOK) - the order must be executed in full immediately, or it is canceled.
+    /// Fill or Kill (FOK) - Executes in its entirety immediately or is canceled if full execution is not possible.
     Fok = 3,
-    /// Good-Till-Date/Time (GTD) - the order is active until a specified date or time.
+    /// Good Till Date (GTD) - Remains active until the specified expiration date or time is reached.
     Gtd = 4,
-    /// Day - the order is active until the end of the current trading session.
+    /// Day - Remains active until the close of the current trading session.
     Day = 5,
-    /// At the Opening (ATO) - the order is scheduled to be executed at the market's opening.
+    /// At the Opening (ATO) - Executes at the market opening or expires if not filled.
     AtTheOpen = 6,
-    /// At the Closing (ATC) - the order is scheduled to be executed at the market's closing.
+    /// At the Closing (ATC) - Executes at the market close or expires if not filled.
     AtTheClose = 7,
 }
 
