@@ -23,6 +23,7 @@ use rust_decimal::Decimal;
 
 use crate::{
     data::order::{BookOrder, OrderId},
+    enums::OrderSideSpecified,
     orderbook::{BookIntegrityError, BookPrice},
     types::{fixed::FIXED_SCALAR, quantity::QuantityRaw},
 };
@@ -61,6 +62,10 @@ impl BookLevel {
         level
     }
 
+    pub fn side(&self) -> OrderSideSpecified {
+        self.price.side
+    }
+
     /// Returns the number of orders at this price level.
     #[must_use]
     pub fn len(&self) -> usize {
@@ -205,6 +210,7 @@ mod tests {
     fn test_empty_level() {
         let level = BookLevel::new(BookPrice::new(Price::from("1.00"), OrderSideSpecified::Buy));
         assert!(level.first().is_none());
+        assert_eq!(level.side(), OrderSideSpecified::Buy);
     }
 
     #[rstest]
