@@ -28,7 +28,7 @@ use rust_decimal::Decimal;
 use ustr::Ustr;
 
 use crate::{
-    enums::{ContingencyType, OrderSide, OrderType, PositionSide, TimeInForce},
+    enums::{ContingencyType, OrderSide, OrderStatus, OrderType, PositionSide, TimeInForce},
     events::OrderInitialized,
     identifiers::{
         AccountId, ClientOrderId, ExecAlgorithmId, InstrumentId, OrderListId, StrategyId, TraderId,
@@ -114,14 +114,22 @@ impl MarketOrder {
         Ok(MarketOrder::from(init))
     }
 
-    #[pyo3(name = "signed_decimal_qty")]
-    fn py_signed_decimal_qty(&self) -> Decimal {
-        self.signed_decimal_qty()
+    #[staticmethod]
+    #[pyo3(name = "opposite_side")]
+    fn py_opposite_side(side: OrderSide) -> OrderSide {
+        OrderCore::opposite_side(side)
     }
 
-    #[pyo3(name = "would_reduce_only")]
-    fn py_would_reduce_only(&self, side: PositionSide, position_qty: Quantity) -> bool {
-        self.would_reduce_only(side, position_qty)
+    #[staticmethod]
+    #[pyo3(name = "closing_side")]
+    fn py_closing_side(side: PositionSide) -> OrderSide {
+        OrderCore::closing_side(side)
+    }
+
+    #[getter]
+    #[pyo3(name = "status")]
+    fn py_status(&self) -> OrderStatus {
+        self.status
     }
 
     #[pyo3(name = "commission")]
@@ -270,18 +278,6 @@ impl MarketOrder {
             .map(|vec| vec.iter().map(|s| s.as_str()).collect())
     }
 
-    #[staticmethod]
-    #[pyo3(name = "opposite_side")]
-    fn py_opposite_side(side: OrderSide) -> OrderSide {
-        OrderCore::opposite_side(side)
-    }
-
-    #[staticmethod]
-    #[pyo3(name = "closing_side")]
-    fn py_closing_side(side: PositionSide) -> OrderSide {
-        OrderCore::closing_side(side)
-    }
-
     #[getter]
     #[pyo3(name = "events")]
     fn py_events(&self, py: Python<'_>) -> PyResult<Vec<PyObject>> {
@@ -291,6 +287,129 @@ impl MarketOrder {
             .collect()
     }
 
+    #[pyo3(name = "signed_decimal_qty")]
+    fn py_signed_decimal_qty(&self) -> Decimal {
+        self.signed_decimal_qty()
+    }
+
+    #[pyo3(name = "would_reduce_only")]
+    fn py_would_reduce_only(&self, side: PositionSide, position_qty: Quantity) -> bool {
+        self.would_reduce_only(side, position_qty)
+    }
+
+    #[pyo3(name = "apply")]
+    fn py_apply(&mut self, event: PyObject, py: Python<'_>) -> PyResult<()> {
+        let event_any = pyobject_to_order_event(py, event).unwrap();
+        self.apply(event_any).map(|_| ()).map_err(to_pyruntime_err)
+    }
+
+    #[staticmethod]
+    #[pyo3(name = "from_dict")]
+    fn py_from_dict(values: &Bound<'_, PyDict>) -> PyResult<Self> {
+        let dict = values.as_ref();
+        let trader_id = TraderId::from(dict.get_item("trader_id")?.extract::<&str>()?);
+        let strategy_id = StrategyId::from(dict.get_item("strategy_id")?.extract::<&str>()?);
+        let instrument_id = InstrumentId::from(dict.get_item("instrument_id")?.extract::<&str>()?);
+        let client_order_id =
+            ClientOrderId::from(dict.get_item("client_order_id")?.extract::<&str>()?);
+        let order_side = dict
+            .get_item("side")?
+            .extract::<&str>()?
+            .parse::<OrderSide>()
+            .unwrap();
+        let quantity = Quantity::from(dict.get_item("quantity")?.extract::<&str>()?);
+        let time_in_force = dict
+            .get_item("time_in_force")?
+            .extract::<&str>()?
+            .parse::<TimeInForce>()
+            .unwrap();
+        let init_id = dict
+            .get_item("init_id")
+            .map(|x| x.extract::<&str>().unwrap().parse::<UUID4>().ok())?
+            .unwrap();
+        let ts_init = dict.get_item("ts_init")?.extract::<u64>()?;
+        let is_reduce_only = dict.get_item("is_reduce_only")?.extract::<bool>()?;
+        let is_quote_quantity = dict.get_item("is_quote_quantity")?.extract::<bool>()?;
+        let contingency_type = dict
+            .get_item("contingency_type")
+            .map(|x| x.extract::<&str>().unwrap().parse::<ContingencyType>().ok())?;
+        let order_list_id = dict.get_item("order_list_id").map(|x| {
+            let extracted_str = x.extract::<&str>();
+            match extracted_str {
+                Ok(item) => Some(OrderListId::from(item)),
+                Err(_) => None,
+            }
+        })?;
+        let linked_order_ids = dict.get_item("linked_order_ids").map(|x| {
+            let extracted_str = x.extract::<Vec<String>>();
+            match extracted_str {
+                Ok(item) => Some(
+                    item.iter()
+                        .map(|x| ClientOrderId::from(x.as_str()))
+                        .collect(),
+                ),
+                Err(_) => None,
+            }
+        })?;
+        let parent_order_id = dict.get_item("parent_order_id").map(|x| {
+            let extracted_str = x.extract::<&str>();
+            match extracted_str {
+                Ok(item) => Some(ClientOrderId::from(item)),
+                Err(_) => None,
+            }
+        })?;
+        let exec_algorithm_id = dict.get_item("exec_algorithm_id").map(|x| {
+            let extracted_str = x.extract::<&str>();
+            match extracted_str {
+                Ok(item) => Some(ExecAlgorithmId::from(item)),
+                Err(_) => None,
+            }
+        })?;
+        let exec_algorithm_params = dict.get_item("exec_algorithm_params").map(|x| {
+            let extracted_str = x.extract::<IndexMap<String, String>>();
+            match extracted_str {
+                Ok(item) => Some(str_indexmap_to_ustr(item)),
+                Err(_) => None,
+            }
+        })?;
+        let exec_spawn_id = dict.get_item("exec_spawn_id").map(|x| {
+            let extracted_str = x.extract::<&str>();
+            match extracted_str {
+                Ok(item) => Some(ClientOrderId::from(item)),
+                Err(_) => None,
+            }
+        })?;
+        let tags = dict.get_item("tags").map(|x| {
+            let extracted_str = x.extract::<Vec<String>>();
+            match extracted_str {
+                Ok(item) => Some(item.iter().map(|s| Ustr::from(s)).collect()),
+                Err(_) => None,
+            }
+        })?;
+        Self::new_checked(
+            trader_id,
+            strategy_id,
+            instrument_id,
+            client_order_id,
+            order_side,
+            quantity,
+            time_in_force,
+            init_id,
+            ts_init.into(),
+            is_reduce_only,
+            is_quote_quantity,
+            contingency_type,
+            order_list_id,
+            linked_order_ids,
+            parent_order_id,
+            exec_algorithm_id,
+            exec_algorithm_params,
+            exec_spawn_id,
+            tags,
+        )
+        .map_err(to_pyvalue_err)
+    }
+
     #[pyo3(name = "to_dict")]
     fn py_to_dict(&self, py: Python<'_>) -> PyResult<PyObject> {
         let dict = PyDict::new(py);
@@ -399,117 +518,4 @@ impl MarketOrder {
         )?;
         Ok(dict.into())
     }
-
-    #[staticmethod]
-    #[pyo3(name = "from_dict")]
-    fn py_from_dict(values: &Bound<'_, PyDict>) -> PyResult<Self> {
-        let dict = values.as_ref();
-        let trader_id = TraderId::from(dict.get_item("trader_id")?.extract::<&str>()?);
-        let strategy_id = StrategyId::from(dict.get_item("strategy_id")?.extract::<&str>()?);
-        let instrument_id = InstrumentId::from(dict.get_item("instrument_id")?.extract::<&str>()?);
-        let client_order_id =
-            ClientOrderId::from(dict.get_item("client_order_id")?.extract::<&str>()?);
-        let order_side = dict
-            .get_item("side")?
-            .extract::<&str>()?
-            .parse::<OrderSide>()
-            .unwrap();
-        let quantity = Quantity::from(dict.get_item("quantity")?.extract::<&str>()?);
-        let time_in_force = dict
-            .get_item("time_in_force")?
-            .extract::<&str>()?
-            .parse::<TimeInForce>()
-            .unwrap();
-        let init_id = dict
-            .get_item("init_id")
-            .map(|x| x.extract::<&str>().unwrap().parse::<UUID4>().ok())?
-            .unwrap();
-        let ts_init = dict.get_item("ts_init")?.extract::<u64>()?;
-        let is_reduce_only = dict.get_item("is_reduce_only")?.extract::<bool>()?;
-        let is_quote_quantity = dict.get_item("is_quote_quantity")?.extract::<bool>()?;
-        let contingency_type = dict
-            .get_item("contingency_type")
-            .map(|x| x.extract::<&str>().unwrap().parse::<ContingencyType>().ok())?;
-        let order_list_id = dict.get_item("order_list_id").map(|x| {
-            let extracted_str = x.extract::<&str>();
-            match extracted_str {
-                Ok(item) => Some(OrderListId::from(item)),
-                Err(_) => None,
-            }
-        })?;
-        let linked_order_ids = dict.get_item("linked_order_ids").map(|x| {
-            let extracted_str = x.extract::<Vec<String>>();
-            match extracted_str {
-                Ok(item) => Some(
-                    item.iter()
-                        .map(|x| ClientOrderId::from(x.as_str()))
-                        .collect(),
-                ),
-                Err(_) => None,
-            }
-        })?;
-        let parent_order_id = dict.get_item("parent_order_id").map(|x| {
-            let extracted_str = x.extract::<&str>();
-            match extracted_str {
-                Ok(item) => Some(ClientOrderId::from(item)),
-                Err(_) => None,
-            }
-        })?;
-        let exec_algorithm_id = dict.get_item("exec_algorithm_id").map(|x| {
-            let extracted_str = x.extract::<&str>();
-            match extracted_str {
-                Ok(item) => Some(ExecAlgorithmId::from(item)),
-                Err(_) => None,
-            }
-        })?;
-        let exec_algorithm_params = dict.get_item("exec_algorithm_params").map(|x| {
-            let extracted_str = x.extract::<IndexMap<String, String>>();
-            match extracted_str {
-                Ok(item) => Some(str_indexmap_to_ustr(item)),
-                Err(_) => None,
-            }
-        })?;
-        let exec_spawn_id = dict.get_item("exec_spawn_id").map(|x| {
-            let extracted_str = x.extract::<&str>();
-            match extracted_str {
-                Ok(item) => Some(ClientOrderId::from(item)),
-                Err(_) => None,
-            }
-        })?;
-        let tags = dict.get_item("tags").map(|x| {
-            let extracted_str = x.extract::<Vec<String>>();
-            match extracted_str {
-                Ok(item) => Some(item.iter().map(|s| Ustr::from(s)).collect()),
-                Err(_) => None,
-            }
-        })?;
-        Self::new_checked(
-            trader_id,
-            strategy_id,
-            instrument_id,
-            client_order_id,
-            order_side,
-            quantity,
-            time_in_force,
-            init_id,
-            ts_init.into(),
-            is_reduce_only,
-            is_quote_quantity,
-            contingency_type,
-            order_list_id,
-            linked_order_ids,
-            parent_order_id,
-            exec_algorithm_id,
-            exec_algorithm_params,
-            exec_spawn_id,
-            tags,
-        )
-        .map_err(to_pyvalue_err)
-    }
-
-    #[pyo3(name = "apply")]
-    fn py_apply(&mut self, event: PyObject, py: Python<'_>) -> PyResult<()> {
-        let event_any = pyobject_to_order_event(py, event).unwrap();
-        self.apply(event_any).map(|_| ()).map_err(to_pyruntime_err)
-    }
 }
