@@ -23,10 +23,6 @@ use std::{
     str::FromStr,
 };
 
-#[cfg(not(feature = "high-precision"))]
-use nautilus_core::correctness::check_positive_u64;
-#[cfg(feature = "high-precision")]
-use nautilus_core::correctness::check_positive_u128;
 use nautilus_core::{
     correctness::{FAILED, check_in_range_inclusive_f64},
     parsing::precision_from_str,
@@ -62,16 +58,6 @@ pub const QUANTITY_MAX: f64 = 18_446_744_073.0;
 /// The minimum valid quantity value which can be represented.
 pub const QUANTITY_MIN: f64 = 0.0;
 
-#[cfg(not(feature = "high-precision"))]
-pub fn check_positive_quantity(value: QuantityRaw, param: &str) -> anyhow::Result<()> {
-    check_positive_u64(value, param)
-}
-
-#[cfg(feature = "high-precision")]
-pub fn check_positive_quantity(value: QuantityRaw, param: &str) -> anyhow::Result<()> {
-    check_positive_u128(value, param)
-}
-
 /// Represents a quantity with a non-negative value.
 ///
 /// Capable of storing either a whole number (no decimal places) of 'contracts'
@@ -470,9 +456,14 @@ impl<'de> Deserialize<'de> for Quantity {
     }
 }
 
-pub fn check_quantity_positive(value: Quantity) -> anyhow::Result<()> {
+/// Checks if the given quantity is positive.
+///
+/// # Errors
+///
+/// Returns an error if the quantity is not positive.
+pub fn check_positive_quantity(value: Quantity, param: &str) -> anyhow::Result<()> {
     if !value.is_positive() {
-        anyhow::bail!("{FAILED}: invalid `Quantity`, should be positive and was {value}")
+        anyhow::bail!("{FAILED}: invalid `Quantity` for '{param}' not positive, was {value}")
     }
     Ok(())
 }
@@ -491,10 +482,10 @@ mod tests {
     use super::*;
 
     #[rstest]
-    #[should_panic(expected = "Condition failed: invalid `Quantity`, should be positive and was 0")]
+    #[should_panic(expected = "Condition failed: invalid `Quantity` for 'qty' not positive, was 0")]
     fn test_check_quantity_positive() {
         let qty = Quantity::new(0.0, 0);
-        check_quantity_positive(qty).unwrap();
+        check_positive_quantity(qty, "qty").unwrap();
     }
 
     #[rstest]
@@ -566,7 +557,7 @@ mod tests {
     #[rstest]
     fn test_check_quantity_positive_ok() {
         let qty = Quantity::new(10.0, 0);
-        check_quantity_positive(qty).unwrap();
+        check_positive_quantity(qty, "qty").unwrap();
     }
 
     #[rstest]
@@ -782,7 +773,7 @@ mod tests {
         assert_eq!(result, "44.12");
     }
 
-    #[test]
+    #[rstest]
     fn test_to_formatted_string() {
         let qty = Quantity::new(1234.5678, 4);
         let formatted = qty.to_formatted_string();
