@@ -18,7 +18,6 @@ use std::{
     time::Duration,
 };
 
-use futures::SinkExt;
 use nautilus_core::python::to_pyvalue_err;
 use pyo3::{create_exception, exceptions::PyException, prelude::*};
 use tokio_tungstenite::tungstenite::{Message, Utf8Bytes};
@@ -26,7 +25,7 @@ use tokio_tungstenite::tungstenite::{Message, Utf8Bytes};
 use crate::{
     mode::ConnectionMode,
     ratelimiter::quota::Quota,
-    websocket::{WebSocketClient, WebSocketConfig},
+    websocket::{Consumer, WebSocketClient, WebSocketConfig, WriterCommand},
 };
 
 // Python exception class for websocket errors
@@ -39,8 +38,8 @@ fn to_websocket_pyerr(e: tokio_tungstenite::tungstenite::Error) -> PyErr {
 #[pymethods]
 impl WebSocketConfig {
     #[new]
-    #[pyo3(signature = (url, handler, headers, heartbeat=None, heartbeat_msg=None, ping_handler=None, reconnect_timeout_ms=10_000, reconnect_delay_initial_ms=2_000, reconnect_delay_max_ms=30_000, reconnect_backoff_factor=1.5, reconnect_jitter_ms=100))]
     #[allow(clippy::too_many_arguments)]
+    #[pyo3(signature = (url, handler, headers, heartbeat=None, heartbeat_msg=None, ping_handler=None, reconnect_timeout_ms=10_000, reconnect_delay_initial_ms=2_000, reconnect_delay_max_ms=30_000, reconnect_backoff_factor=1.5, reconnect_jitter_ms=100))]
     fn py_new(
         url: String,
         handler: PyObject,
@@ -56,7 +55,7 @@ impl WebSocketConfig {
     ) -> Self {
         Self {
             url,
-            handler: Some(Arc::new(handler)),
+            handler: Consumer::Python(Some(Arc::new(handler))),
             headers,
             heartbeat,
             heartbeat_msg,
@@ -179,18 +178,18 @@ impl WebSocketClient {
         py: Python<'py>,
         keys: Option<Vec<String>>,
     ) -> PyResult<Bound<'py, PyAny>> {
-        let writer = slf.writer.clone();
         let rate_limiter = slf.rate_limiter.clone();
+        let writer_tx = slf.writer_tx.clone();
 
         pyo3_async_runtimes::tokio::future_into_py(py, async move {
             rate_limiter.await_keys_ready(keys).await;
             tracing::trace!("Sending binary: {data:?}");
 
-            let mut guard = writer.lock().await;
-            guard
-                .send(Message::Binary(data.into()))
-                .await
-                .map_err(to_websocket_pyerr)
+            let msg = Message::Binary(data.into());
+            if let Err(e) = writer_tx.send(WriterCommand::Send(msg)) {
+                tracing::error!("{e}");
+            }
+            Ok(())
         })
     }
 
@@ -217,18 +216,18 @@ impl WebSocketClient {
     ) -> PyResult<Bound<'py, PyAny>> {
         let data_str = String::from_utf8(data).map_err(to_pyvalue_err)?;
         let data = Utf8Bytes::from(data_str);
-        let writer = slf.writer.clone();
         let rate_limiter = slf.rate_limiter.clone();
+        let writer_tx = slf.writer_tx.clone();
 
         pyo3_async_runtimes::tokio::future_into_py(py, async move {
             rate_limiter.await_keys_ready(keys).await;
             tracing::trace!("Sending text: {data}");
 
-            let mut guard = writer.lock().await;
-            guard
-                .send(Message::Text(data))
-                .await
-                .map_err(to_websocket_pyerr)
+            let msg = Message::Text(data);
+            if let Err(e) = writer_tx.send(WriterCommand::Send(msg)) {
+                tracing::error!("{e}");
+            }
+            Ok(())
         })
     }
 
@@ -244,15 +243,15 @@ impl WebSocketClient {
         py: Python<'py>,
     ) -> PyResult<Bound<'py, PyAny>> {
         let data_str = String::from_utf8(data.clone()).map_err(to_pyvalue_err)?;
-        let writer = slf.writer.clone();
+        let writer_tx = slf.writer_tx.clone();
         tracing::trace!("Sending pong: {data_str}");
 
         pyo3_async_runtimes::tokio::future_into_py(py, async move {
-            let mut guard = writer.lock().await;
-            guard
-                .send(Message::Pong(data.into()))
-                .await
-                .map_err(to_websocket_pyerr)
+            let msg = Message::Pong(data.into());
+            if let Err(e) = writer_tx.send(WriterCommand::Send(msg)) {
+                tracing::error!("{e}");
+            }
+            Ok(())
         })
     }
 }
