@@ -16,6 +16,18 @@
 //! High-performance raw TCP client implementation with TLS capability, automatic reconnection
 //! with exponential backoff and state management.
 
+//! **Key features**:
+//! - Connection state tracking (ACTIVE/RECONNECTING/DISCONNECTING/CLOSED)
+//! - Synchronized reconnection with backoff
+//! - Split read/write architecture
+//! - Python callback integration
+//!
+//! **Design**:
+//! - Single reader, multiple writer model
+//! - Read half runs in dedicated task
+//! - Write half runs in dedicated task connected with channel
+//! - Controller task manages lifecycle
+
 use std::{
     path::Path,
     sync::{
@@ -25,12 +37,12 @@ use std::{
     time::Duration,
 };
 
+use bytes::Bytes;
 use nautilus_cryptography::providers::install_cryptographic_provider;
 use pyo3::prelude::*;
 use tokio::{
     io::{AsyncReadExt, AsyncWriteExt, ReadHalf, WriteHalf},
     net::TcpStream,
-    sync::Mutex,
 };
 use tokio_tungstenite::{
     MaybeTlsStream,
@@ -39,13 +51,14 @@ use tokio_tungstenite::{
 
 use crate::{
     backoff::ExponentialBackoff,
+    fix::process_fix_buffer,
     mode::ConnectionMode,
     tls::{Connector, create_tls_config_from_certs_dir, tcp_tls},
 };
 
 type TcpWriter = WriteHalf<MaybeTlsStream<TcpStream>>;
-type SharedTcpWriter = Arc<Mutex<WriteHalf<MaybeTlsStream<TcpStream>>>>;
 type TcpReader = ReadHalf<MaybeTlsStream<TcpStream>>;
+pub type TcpMessageHandler = dyn Fn(&[u8]) + Send + Sync;
 
 /// Configuration for TCP socket connection.
 #[derive(Debug, Clone)]
@@ -60,8 +73,8 @@ pub struct SocketConfig {
     pub mode: Mode,
     /// The sequence of bytes which separates lines.
     pub suffix: Vec<u8>,
-    /// The Python function to handle incoming messages.
-    pub handler: Arc<PyObject>,
+    /// The optional Python function to handle incoming messages.
+    pub py_handler: Option<Arc<PyObject>>,
     /// The optional heartbeat with period and beat message.
     pub heartbeat: Option<(u64, Vec<u8>)>,
     /// The timeout (milliseconds) for reconnection attempts.
@@ -78,14 +91,23 @@ pub struct SocketConfig {
     pub certs_dir: Option<String>,
 }
 
+/// Represents a command for the writer task.
+#[derive(Debug)]
+pub enum WriterCommand {
+    /// Update the writer reference with a new one after reconnection.
+    Update(TcpWriter),
+    /// Send data to the server.
+    Send(Bytes),
+}
+
 /// Creates a TcpStream with the server.
 ///
 /// The stream can be encrypted with TLS or Plain. The stream is split into
 /// read and write ends:
 /// - The read end is passed to the task that keeps receiving
 ///   messages from the server and passing them to a handler.
-/// - The write end is wrapped in an `Arc<Mutex>` and used to send messages
-///   or heart beats.
+/// - The write end is passed to a task which receives messages over a channel
+///   to send to the server.
 ///
 /// The heartbeat is optional and can be configured with an interval and data to
 /// send.
@@ -101,15 +123,20 @@ struct SocketClientInner {
     config: SocketConfig,
     connector: Option<Connector>,
     read_task: Arc<tokio::task::JoinHandle<()>>,
+    write_task: tokio::task::JoinHandle<()>,
+    writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
     heartbeat_task: Option<tokio::task::JoinHandle<()>>,
-    writer: SharedTcpWriter,
     connection_mode: Arc<AtomicU8>,
     reconnect_timeout: Duration,
     backoff: ExponentialBackoff,
+    handler: Option<Arc<TcpMessageHandler>>,
 }
 
 impl SocketClientInner {
-    pub async fn connect_url(config: SocketConfig) -> anyhow::Result<Self> {
+    pub async fn connect_url(
+        config: SocketConfig,
+        handler: Option<Arc<TcpMessageHandler>>,
+    ) -> anyhow::Result<Self> {
         install_cryptographic_provider();
 
         let SocketConfig {
@@ -117,7 +144,7 @@ impl SocketClientInner {
             mode,
             heartbeat,
             suffix,
-            handler,
+            py_handler,
             reconnect_timeout_ms,
             reconnect_delay_initial_ms,
             reconnect_delay_max_ms,
@@ -133,20 +160,29 @@ impl SocketClientInner {
         };
 
         let (reader, writer) = Self::tls_connect_with_server(url, *mode, connector.clone()).await?;
-        let writer = Arc::new(Mutex::new(writer));
+        tracing::debug!("Connected");
 
         let connection_mode = Arc::new(AtomicU8::new(ConnectionMode::Active.as_u8()));
 
-        let handler = Python::with_gil(|py| handler.clone_ref(py));
-        let read_task = Arc::new(Self::spawn_read_task(reader, handler, suffix.clone()));
+        let read_task = Arc::new(Self::spawn_read_task(
+            connection_mode.clone(),
+            reader,
+            handler.clone(),
+            py_handler.clone(),
+            suffix.clone(),
+        ));
+
+        let (writer_tx, writer_rx) = tokio::sync::mpsc::unbounded_channel::<WriterCommand>();
+
+        let write_task =
+            Self::spawn_write_task(connection_mode.clone(), writer, writer_rx, suffix.clone());
 
         // Optionally spawn a heartbeat task to periodically ping server
         let heartbeat_task = heartbeat.as_ref().map(|heartbeat| {
             Self::spawn_heartbeat_task(
                 connection_mode.clone(),
                 heartbeat.clone(),
-                writer.clone(),
-                suffix.clone(),
+                writer_tx.clone(),
             )
         });
 
@@ -163,11 +199,13 @@ impl SocketClientInner {
             config,
             connector,
             read_task,
+            write_task,
+            writer_tx,
             heartbeat_task,
-            writer,
             connection_mode,
             reconnect_timeout,
             backoff,
+            handler,
         })
     }
 
@@ -176,37 +214,38 @@ impl SocketClientInner {
         mode: Mode,
         connector: Option<Connector>,
     ) -> Result<(TcpReader, TcpWriter), Error> {
-        tracing::debug!("Connecting to server");
-        let stream = TcpStream::connect(url).await?;
-        tracing::debug!("Making TLS connection");
-        let request = url.into_client_request()?;
-        tcp_tls(&request, mode, stream, connector)
-            .await
-            .map(tokio::io::split)
+        tracing::debug!("Connecting to {url}");
+        let tcp_result = TcpStream::connect(url).await;
+
+        match tcp_result {
+            Ok(stream) => {
+                tracing::debug!("TCP connection established, proceeding with TLS");
+                let request = url.into_client_request()?;
+                tcp_tls(&request, mode, stream, connector)
+                    .await
+                    .map(tokio::io::split)
+            }
+            Err(e) => {
+                tracing::error!("TCP connection failed: {e:?}");
+                Err(Error::Io(e))
+            }
+        }
     }
 
     /// Reconnect with server.
     ///
-    /// Make a new connection with server. Use the new read and write halves
-    /// to update the shared writer and the read and heartbeat tasks.
+    /// Makes a new connection with server, uses the new read and write halves
+    /// to update the reader and writer.
     async fn reconnect(&mut self) -> Result<(), Error> {
         tracing::debug!("Reconnecting");
 
         tokio::time::timeout(self.reconnect_timeout, async {
-            // Clean up existing tasks
-            shutdown(
-                self.read_task.clone(),
-                self.heartbeat_task.take(),
-                self.writer.clone(),
-            )
-            .await;
-
             let SocketConfig {
                 url,
                 mode,
-                heartbeat,
+                heartbeat: _,
                 suffix,
-                handler,
+                py_handler,
                 reconnect_timeout_ms: _,
                 reconnect_delay_initial_ms: _,
                 reconnect_backoff_factor: _,
@@ -216,31 +255,33 @@ impl SocketClientInner {
             } = &self.config;
             // Create a fresh connection
             let connector = self.connector.clone();
-            let (reader, writer) = Self::tls_connect_with_server(url, *mode, connector).await?;
-            let writer = Arc::new(Mutex::new(writer));
-            self.writer = writer.clone();
+            let (reader, new_writer) = Self::tls_connect_with_server(url, *mode, connector).await?;
+            tracing::debug!("Connected");
 
-            // Spawn new read task
-            let handler_for_read = Python::with_gil(|py| handler.clone_ref(py));
-            self.read_task = Arc::new(Self::spawn_read_task(
-                reader,
-                handler_for_read,
-                suffix.clone(),
-            ));
+            if let Err(e) = self.writer_tx.send(WriterCommand::Update(new_writer)) {
+                tracing::error!("{e}");
+            }
 
-            // Optionally spawn new heartbeat task
-            self.heartbeat_task = heartbeat.as_ref().map(|heartbeat| {
-                Self::spawn_heartbeat_task(
-                    self.connection_mode.clone(),
-                    heartbeat.clone(),
-                    writer.clone(),
-                    suffix.clone(),
-                )
-            });
+            // Delay before closing connection
+            tokio::time::sleep(Duration::from_millis(100)).await;
+
+            if !self.read_task.is_finished() {
+                self.read_task.abort();
+                tracing::debug!("Aborted task 'read'");
+            }
 
             self.connection_mode
                 .store(ConnectionMode::Active.as_u8(), Ordering::SeqCst);
 
+            // Spawn new read task
+            self.read_task = Arc::new(Self::spawn_read_task(
+                self.connection_mode.clone(),
+                reader,
+                self.handler.clone(),
+                py_handler.clone(),
+                suffix.clone(),
+            ));
+
             tracing::debug!("Reconnect succeeded");
             Ok(())
         })
@@ -270,77 +311,182 @@ impl SocketClientInner {
 
     #[must_use]
     fn spawn_read_task(
+        connection_state: Arc<AtomicU8>,
         mut reader: TcpReader,
-        handler: PyObject,
+        handler: Option<Arc<TcpMessageHandler>>,
+        py_handler: Option<Arc<PyObject>>,
         suffix: Vec<u8>,
     ) -> tokio::task::JoinHandle<()> {
         tracing::debug!("Started task 'read'");
 
+        // Interval between checking the connection mode
+        let check_interval = Duration::from_millis(10);
+
         tokio::task::spawn(async move {
             let mut buf = Vec::new();
 
             loop {
-                match reader.read_buf(&mut buf).await {
+                if !ConnectionMode::from_atomic(&connection_state).is_active() {
+                    break;
+                }
+
+                match tokio::time::timeout(check_interval, reader.read_buf(&mut buf)).await {
                     // Connection has been terminated or vector buffer is complete
-                    Ok(0) => {
+                    Ok(Ok(0)) => {
                         tracing::debug!("Connection closed by server");
                         break;
                     }
-                    Err(e) => {
+                    Ok(Err(e)) => {
                         tracing::debug!("Connection ended: {e}");
                         break;
                     }
                     // Received bytes of data
-                    Ok(bytes) => {
+                    Ok(Ok(bytes)) => {
                         tracing::trace!("Received <binary> {bytes} bytes");
 
-                        // While received data has a line break
-                        // drain it and pass it to the handler
-                        while let Some((i, _)) = &buf
-                            .windows(suffix.len())
-                            .enumerate()
-                            .find(|(_, pair)| pair.eq(&suffix))
-                        {
-                            let mut data: Vec<u8> = buf.drain(0..i + suffix.len()).collect();
-                            data.truncate(data.len() - suffix.len());
-
-                            if let Err(e) =
-                                Python::with_gil(|py| handler.call1(py, (data.as_slice(),)))
+                        if let Some(handler) = &handler {
+                            process_fix_buffer(&mut buf, handler);
+                        } else {
+                            while let Some((i, _)) = &buf
+                                .windows(suffix.len())
+                                .enumerate()
+                                .find(|(_, pair)| pair.eq(&suffix))
                             {
-                                tracing::error!("Call to handler failed: {e}");
-                                break;
+                                let mut data: Vec<u8> = buf.drain(0..i + suffix.len()).collect();
+                                data.truncate(data.len() - suffix.len());
+
+                                if let Some(handler) = &handler {
+                                    handler(&data);
+                                }
+
+                                if let Some(py_handler) = &py_handler {
+                                    if let Err(e) = Python::with_gil(|py| {
+                                        py_handler.call1(py, (data.as_slice(),))
+                                    }) {
+                                        tracing::error!("Call to handler failed: {e}");
+                                        break;
+                                    }
+                                }
                             }
                         }
                     }
-                };
+                    Err(_) => {
+                        // Timeout - continue loop and check connection mode
+                        continue;
+                    }
+                }
             }
 
             tracing::debug!("Completed task 'read'");
         })
     }
 
+    fn spawn_write_task(
+        connection_state: Arc<AtomicU8>,
+        writer: TcpWriter,
+        mut writer_rx: tokio::sync::mpsc::UnboundedReceiver<WriterCommand>,
+        suffix: Vec<u8>,
+    ) -> tokio::task::JoinHandle<()> {
+        tracing::debug!("Started task 'write'");
+
+        // Interval between checking the connection mode
+        let check_interval = Duration::from_millis(10);
+
+        tokio::task::spawn(async move {
+            let mut active_writer = writer;
+
+            loop {
+                if matches!(
+                    ConnectionMode::from_atomic(&connection_state),
+                    ConnectionMode::Disconnect | ConnectionMode::Closed
+                ) {
+                    break;
+                }
+
+                match tokio::time::timeout(check_interval, writer_rx.recv()).await {
+                    Ok(Some(msg)) => {
+                        // Re-check connection mode after receiving a message
+                        let mode = ConnectionMode::from_atomic(&connection_state);
+                        if matches!(mode, ConnectionMode::Disconnect | ConnectionMode::Closed) {
+                            break;
+                        }
+
+                        match msg {
+                            WriterCommand::Update(new_writer) => {
+                                tracing::debug!("Received new writer");
+
+                                // Delay before closing connection
+                                tokio::time::sleep(Duration::from_millis(100)).await;
+
+                                // Attempt to shutdown the writer gracefully before updating,
+                                // we ignore any error as the writer may already be closed.
+                                _ = active_writer.shutdown().await;
+
+                                active_writer = new_writer;
+                                tracing::debug!("Updated writer");
+                            }
+                            _ if mode.is_reconnect() => {
+                                tracing::warn!("Skipping message while reconnecting, {msg:?}");
+                                continue;
+                            }
+                            WriterCommand::Send(msg) => {
+                                if let Err(e) = active_writer.write_all(&msg).await {
+                                    tracing::error!("Failed to send message: {e}");
+                                    // Mode is active so trigger reconnection
+                                    tracing::warn!("Writer triggering reconnect");
+                                    connection_state
+                                        .store(ConnectionMode::Reconnect.as_u8(), Ordering::SeqCst);
+                                    continue;
+                                }
+                                if let Err(e) = active_writer.write_all(&suffix).await {
+                                    tracing::error!("Failed to send message: {e}");
+                                }
+                            }
+                        }
+                    }
+                    Ok(None) => {
+                        // Channel closed - writer task should terminate
+                        tracing::debug!("Writer channel closed, terminating writer task");
+                        break;
+                    }
+                    Err(_) => {
+                        // Timeout - just continue the loop
+                        continue;
+                    }
+                }
+            }
+
+            // Attempt to shutdown the writer gracefully before exiting,
+            // we ignore any error as the writer may already be closed.
+            _ = active_writer.shutdown().await;
+
+            tracing::debug!("Completed task 'write'");
+        })
+    }
+
     fn spawn_heartbeat_task(
         connection_state: Arc<AtomicU8>,
         heartbeat: (u64, Vec<u8>),
-        writer: SharedTcpWriter,
-        suffix: Vec<u8>,
+        writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
     ) -> tokio::task::JoinHandle<()> {
         tracing::debug!("Started task 'heartbeat'");
-        let (interval_secs, mut message) = heartbeat;
+        let (interval_secs, message) = heartbeat;
 
         tokio::task::spawn(async move {
             let interval = Duration::from_secs(interval_secs);
-            message.extend(suffix);
 
             loop {
                 tokio::time::sleep(interval).await;
 
                 match ConnectionMode::from_u8(connection_state.load(Ordering::SeqCst)) {
                     ConnectionMode::Active => {
-                        let mut guard = writer.lock().await;
-                        match guard.write_all(&message).await {
-                            Ok(()) => tracing::trace!("Sent heartbeat"),
-                            Err(e) => tracing::error!("Failed to send heartbeat: {e}"),
+                        let msg = WriterCommand::Send(message.clone().into());
+
+                        match writer_tx.send(msg) {
+                            Ok(()) => tracing::trace!("Sent heartbeat to writer task"),
+                            Err(e) => {
+                                tracing::error!("Failed to send heartbeat to writer task: {e}");
+                            }
                         }
                     }
                     ConnectionMode::Reconnect => continue,
@@ -353,61 +499,22 @@ impl SocketClientInner {
     }
 }
 
-/// Shutdown socket connection.
-///
-/// The client must be explicitly shutdown before dropping otherwise
-/// the connection might still be alive for some time before terminating.
-/// Closing the connection is an async call which cannot be done by the
-/// drop method so it must be done explicitly.
-async fn shutdown(
-    read_task: Arc<tokio::task::JoinHandle<()>>,
-    heartbeat_task: Option<tokio::task::JoinHandle<()>>,
-    writer: SharedTcpWriter,
-) {
-    tracing::debug!("Shutting down inner client");
-
-    let timeout = Duration::from_secs(5);
-    if tokio::time::timeout(timeout, async {
-        // Final close of writer
-        let mut writer = writer.lock().await;
-        if let Err(e) = writer.shutdown().await {
-            tracing::error!("Error on shutdown: {e}");
-        }
-        drop(writer);
-
-        tokio::time::sleep(Duration::from_millis(100)).await;
-
-        // Abort tasks
-        if !read_task.is_finished() {
-            read_task.abort();
-            tracing::debug!("Aborted read task");
-        }
-        if let Some(task) = heartbeat_task {
-            if !task.is_finished() {
-                task.abort();
-                tracing::debug!("Aborted heartbeat task");
-            }
-        }
-    })
-    .await
-    .is_err()
-    {
-        tracing::error!("Shutdown timed out after {}s", timeout.as_secs());
-    }
-
-    tracing::debug!("Closed");
-}
-
 impl Drop for SocketClientInner {
     fn drop(&mut self) {
         if !self.read_task.is_finished() {
             self.read_task.abort();
+            tracing::debug!("Aborted task 'read'");
+        }
+
+        if !self.write_task.is_finished() {
+            self.write_task.abort();
+            tracing::debug!("Aborted task 'write'");
         }
 
-        // Cancel heart beat task
         if let Some(ref handle) = self.heartbeat_task.take() {
             if !handle.is_finished() {
                 handle.abort();
+                tracing::debug!("Aborted task 'heartbeat'");
             }
         }
     }
@@ -418,10 +525,9 @@ impl Drop for SocketClientInner {
     pyo3::pyclass(module = "nautilus_trader.core.nautilus_pyo3.network")
 )]
 pub struct SocketClient {
-    pub(crate) writer: SharedTcpWriter,
     pub(crate) controller_task: tokio::task::JoinHandle<()>,
     pub(crate) connection_mode: Arc<AtomicU8>,
-    pub(crate) suffix: Vec<u8>,
+    pub writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
 }
 
 impl SocketClient {
@@ -432,13 +538,13 @@ impl SocketClient {
     /// Returns any error connecting to the server.
     pub async fn connect(
         config: SocketConfig,
+        handler: Option<Arc<TcpMessageHandler>>,
         post_connection: Option<PyObject>,
         post_reconnection: Option<PyObject>,
         post_disconnection: Option<PyObject>,
     ) -> anyhow::Result<Self> {
-        let suffix = config.suffix.clone();
-        let inner = SocketClientInner::connect_url(config).await?;
-        let writer = inner.writer.clone();
+        let inner = SocketClientInner::connect_url(config, handler).await?;
+        let writer_tx = inner.writer_tx.clone();
         let connection_mode = inner.connection_mode.clone();
 
         let controller_task = Self::spawn_controller_task(
@@ -456,10 +562,9 @@ impl SocketClient {
         }
 
         Ok(Self {
-            writer,
             controller_task,
             connection_mode,
-            suffix,
+            writer_tx,
         })
     }
 
@@ -543,7 +648,7 @@ impl SocketClient {
     /// # Errors
     ///
     /// Returns any I/O error.
-    pub async fn send_bytes(&self, data: &[u8]) -> Result<(), std::io::Error> {
+    pub async fn send_bytes(&self, data: Vec<u8>) -> Result<(), std::io::Error> {
         if self.is_closed() {
             return Err(std::io::Error::new(
                 std::io::ErrorKind::NotConnected,
@@ -574,22 +679,27 @@ impl SocketClient {
             {
                 Ok(Ok(())) => tracing::debug!("Client now active"),
                 Ok(Err(e)) => {
-                    tracing::error!("Cannot send data ({}): {e}", String::from_utf8_lossy(data));
+                    tracing::error!(
+                        "Failed to send data ({}): {e}",
+                        String::from_utf8_lossy(&data)
+                    );
                     return Ok(());
                 }
                 Err(_) => {
                     tracing::error!(
-                        "Cannot send data ({}): timeout waiting to become ACTIVE",
-                        String::from_utf8_lossy(data)
+                        "Failed to send data ({}): timeout waiting to become ACTIVE",
+                        String::from_utf8_lossy(&data)
                     );
                     return Ok(());
                 }
             }
         }
 
-        let mut writer = self.writer.lock().await;
-        writer.write_all(data).await?;
-        writer.write_all(&self.suffix).await
+        let msg = WriterCommand::Send(data.into());
+        if let Err(e) = self.writer_tx.send(msg) {
+            tracing::error!("{e}");
+        }
+        Ok(())
     }
 
     fn spawn_controller_task(
@@ -609,12 +719,31 @@ impl SocketClient {
 
                 if mode.is_disconnect() {
                     tracing::debug!("Disconnecting");
-                    shutdown(
-                        inner.read_task.clone(),
-                        inner.heartbeat_task.take(),
-                        inner.writer.clone(),
-                    )
-                    .await;
+
+                    let timeout = Duration::from_secs(5);
+                    if tokio::time::timeout(timeout, async {
+                        // Delay awaiting graceful shutdown
+                        tokio::time::sleep(Duration::from_millis(100)).await;
+
+                        if !inner.read_task.is_finished() {
+                            inner.read_task.abort();
+                            tracing::debug!("Aborted task 'read'");
+                        }
+
+                        if let Some(task) = &inner.heartbeat_task {
+                            if !task.is_finished() {
+                                task.abort();
+                                tracing::debug!("Aborted task 'heartbeat'");
+                            }
+                        }
+                    })
+                    .await
+                    .is_err()
+                    {
+                        tracing::error!("Shutdown timed out after {}s", timeout.as_secs());
+                    }
+
+                    tracing::debug!("Closed");
 
                     if let Some(ref handler) = post_disconnection {
                         Python::with_gil(|py| match handler.call0(py) {
@@ -635,7 +764,9 @@ impl SocketClient {
 
                             if let Some(ref handler) = post_reconnection {
                                 Python::with_gil(|py| match handler.call0(py) {
-                                    Ok(_) => tracing::debug!("Called `post_reconnection` handler"),
+                                    Ok(_) => {
+                                        tracing::debug!("Called `post_reconnection` handler");
+                                    }
                                     Err(e) => tracing::error!(
                                         "Error calling `post_reconnection` handler: {e}"
                                     ),
@@ -644,7 +775,7 @@ impl SocketClient {
                         }
                         Err(e) => {
                             let duration = inner.backoff.next_duration();
-                            tracing::warn!("Reconnect attempt failed: {e}",);
+                            tracing::warn!("Reconnect attempt failed: {e}");
                             if !duration.is_zero() {
                                 tracing::warn!("Backing off for {}s...", duration.as_secs_f64());
                             }
@@ -656,6 +787,8 @@ impl SocketClient {
             inner
                 .connection_mode
                 .store(ConnectionMode::Closed.as_u8(), Ordering::SeqCst);
+
+            tracing::debug!("Completed task 'controller'");
         })
     }
 }
@@ -674,6 +807,7 @@ mod tests {
     use tokio::{
         io::{AsyncReadExt, AsyncWriteExt},
         net::{TcpListener, TcpStream},
+        sync::Mutex,
         task,
         time::{Duration, sleep},
     };
@@ -771,7 +905,7 @@ counter = Counter()
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler: Arc::new(create_handler()),
+            py_handler: Some(Arc::new(create_handler())),
             heartbeat: None,
             reconnect_timeout_ms: None,
             reconnect_delay_initial_ms: None,
@@ -781,17 +915,17 @@ counter = Counter()
             certs_dir: None,
         };
 
-        let client = SocketClient::connect(config, None, None, None)
+        let client = SocketClient::connect(config, None, None, None, None)
             .await
             .expect("Client connect failed unexpectedly");
 
-        client.send_bytes(b"Hello").await.unwrap();
-        client.send_bytes(b"World").await.unwrap();
+        client.send_bytes(b"Hello".into()).await.unwrap();
+        client.send_bytes(b"World".into()).await.unwrap();
 
         // Wait a bit for the server to echo them back
         sleep(Duration::from_millis(100)).await;
 
-        client.send_bytes(b"close").await.unwrap();
+        client.send_bytes(b"close".into()).await.unwrap();
         server_task.await.unwrap();
         assert!(!client.is_closed());
     }
@@ -807,7 +941,7 @@ counter = Counter()
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler: Arc::new(create_handler()),
+            py_handler: Some(Arc::new(create_handler())),
             heartbeat: None,
             reconnect_timeout_ms: None,
             reconnect_delay_initial_ms: None,
@@ -817,7 +951,7 @@ counter = Counter()
             certs_dir: None,
         };
 
-        let client_res = SocketClient::connect(config, None, None, None).await;
+        let client_res = SocketClient::connect(config, None, None, None, None).await;
         assert!(
             client_res.is_err(),
             "Should fail quickly with no server listening"
@@ -843,7 +977,7 @@ counter = Counter()
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler: Arc::new(create_handler()),
+            py_handler: Some(Arc::new(create_handler())),
             heartbeat: None,
             reconnect_timeout_ms: None,
             reconnect_delay_initial_ms: None,
@@ -853,7 +987,7 @@ counter = Counter()
             certs_dir: None,
         };
 
-        let client = SocketClient::connect(config, None, None, None)
+        let client = SocketClient::connect(config, None, None, None, None)
             .await
             .unwrap();
 
@@ -898,7 +1032,7 @@ counter = Counter()
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler: Arc::new(create_handler()),
+            py_handler: Some(Arc::new(create_handler())),
             heartbeat,
             reconnect_timeout_ms: None,
             reconnect_delay_initial_ms: None,
@@ -908,7 +1042,7 @@ counter = Counter()
             certs_dir: None,
         };
 
-        let client = SocketClient::connect(config, None, None, None)
+        let client = SocketClient::connect(config, None, None, None, None)
             .await
             .unwrap();
 
@@ -952,17 +1086,17 @@ def handler(bytes_data):
         let filename = CString::new("test".to_string()).unwrap();
         let module = CString::new("test".to_string()).unwrap();
 
-        let handler = Python::with_gil(|py| {
+        let py_handler = Some(Python::with_gil(|py| {
             let pymod = PyModule::from_code(py, &code, &filename, &module).unwrap();
             let func = pymod.getattr("handler").unwrap();
             Arc::new(func.into_py_any_unwrap(py))
-        });
+        }));
 
         let config = SocketConfig {
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler,
+            py_handler,
             heartbeat: None,
             reconnect_timeout_ms: None,
             reconnect_delay_initial_ms: None,
@@ -972,14 +1106,14 @@ def handler(bytes_data):
             certs_dir: None,
         };
 
-        let client = SocketClient::connect(config, None, None, None)
+        let client = SocketClient::connect(config, None, None, None, None)
             .await
             .expect("Client connect failed unexpectedly");
 
-        client.send_bytes(b"hello").await.unwrap();
+        client.send_bytes(b"hello".into()).await.unwrap();
         sleep(Duration::from_millis(100)).await;
 
-        client.send_bytes(b"ERR").await.unwrap();
+        client.send_bytes(b"ERR".into()).await.unwrap();
         sleep(Duration::from_secs(1)).await;
 
         assert!(client.is_active());
@@ -1019,7 +1153,7 @@ def handler(bytes_data):
             url: format!("127.0.0.1:{port}"),
             mode: Mode::Plain,
             suffix: b"\r\n".to_vec(),
-            handler: Arc::new(create_handler()),
+            py_handler: Some(Arc::new(create_handler())),
             heartbeat: None,
             reconnect_timeout_ms: Some(5_000),
             reconnect_delay_initial_ms: Some(500),
@@ -1029,7 +1163,7 @@ def handler(bytes_data):
             certs_dir: None,
         };
 
-        let client = SocketClient::connect(config, None, None, None)
+        let client = SocketClient::connect(config, None, None, None, None)
             .await
             .expect("Client connect failed unexpectedly");
 
@@ -1041,7 +1175,7 @@ def handler(bytes_data):
         wait_until_async(|| async { client.is_active() }, Duration::from_secs(10)).await;
 
         client
-            .send_bytes(b"TestReconnect")
+            .send_bytes(b"TestReconnect".into())
             .await
             .expect("Send failed");
 
