@@ -19,14 +19,13 @@
 //! **Key features**:
 //! - Connection state tracking (ACTIVE/RECONNECTING/DISCONNECTING/CLOSED)
 //! - Synchronized reconnection with backoff
-//! - Clean shutdown sequence
 //! - Split read/write architecture
 //! - Python callback integration
 //!
 //! **Design**:
 //! - Single reader, multiple writer model
 //! - Read half runs in dedicated task
-//! - Write half protected by `Arc<Mutex>`
+//! - Write half runs in dedicated task connected with channel
 //! - Controller task manages lifecycle
 
 use std::{
@@ -44,7 +43,10 @@ use futures_util::{
 use http::HeaderName;
 use nautilus_cryptography::providers::install_cryptographic_provider;
 use pyo3::{prelude::*, types::PyBytes};
-use tokio::{net::TcpStream, sync::Mutex};
+use tokio::{
+    net::TcpStream,
+    sync::mpsc::{self, Receiver, Sender},
+};
 use tokio_tungstenite::{
     MaybeTlsStream, WebSocketStream, connect_async,
     tungstenite::{Error, Message, client::IntoClientRequest, http::HeaderValue},
@@ -56,10 +58,22 @@ use crate::{
     ratelimiter::{RateLimiter, clock::MonotonicClock, quota::Quota},
 };
 type MessageWriter = SplitSink<WebSocketStream<MaybeTlsStream<TcpStream>>, Message>;
-type SharedMessageWriter =
-    Arc<Mutex<SplitSink<WebSocketStream<MaybeTlsStream<TcpStream>>, Message>>>;
 pub type MessageReader = SplitStream<WebSocketStream<MaybeTlsStream<TcpStream>>>;
 
+#[derive(Debug, Clone)]
+pub enum Consumer {
+    Python(Option<Arc<PyObject>>),
+    Rust(Sender<Message>),
+}
+
+impl Consumer {
+    #[must_use]
+    pub fn rust_consumer() -> (Self, Receiver<Message>) {
+        let (tx, rx) = mpsc::channel(100);
+        (Self::Rust(tx), rx)
+    }
+}
+
 #[derive(Debug, Clone)]
 #[cfg_attr(
     feature = "python",
@@ -71,7 +85,7 @@ pub struct WebSocketConfig {
     /// The default headers.
     pub headers: Vec<(String, String)>,
     /// The Python function to handle incoming messages.
-    pub handler: Option<Arc<PyObject>>,
+    pub handler: Consumer,
     /// The optional heartbeat interval (seconds).
     pub heartbeat: Option<u64>,
     /// The optional heartbeat message.
@@ -90,6 +104,15 @@ pub struct WebSocketConfig {
     pub reconnect_jitter_ms: Option<u64>,
 }
 
+/// Represents a command for the writer task.
+#[derive(Debug)]
+pub(crate) enum WriterCommand {
+    /// Update the writer reference with a new one after reconnection.
+    Update(MessageWriter),
+    /// Send message to the server.
+    Send(Message),
+}
+
 /// `WebSocketClient` connects to a websocket server to read and send messages.
 ///
 /// The client is opinionated about how messages are read and written. It
@@ -108,8 +131,9 @@ pub struct WebSocketConfig {
 struct WebSocketClientInner {
     config: WebSocketConfig,
     read_task: Option<tokio::task::JoinHandle<()>>,
+    write_task: tokio::task::JoinHandle<()>,
+    writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
     heartbeat_task: Option<tokio::task::JoinHandle<()>>,
-    writer: SharedMessageWriter,
     connection_mode: Arc<AtomicU8>,
     reconnect_timeout: Duration,
     backoff: ExponentialBackoff,
@@ -135,14 +159,27 @@ impl WebSocketClientInner {
             reconnect_jitter_ms,
         } = &config;
         let (writer, reader) = Self::connect_with_server(url, headers.clone()).await?;
-        let writer = Arc::new(Mutex::new(writer));
 
         let connection_mode = Arc::new(AtomicU8::new(ConnectionMode::Active.as_u8()));
 
-        // Only spawn read task if handler is provided
-        let read_task = handler
-            .as_ref()
-            .map(|handler| Self::spawn_read_task(reader, handler.clone(), ping_handler.clone()));
+        let read_task = match &handler {
+            Consumer::Python(handler) => handler.as_ref().map(|handler| {
+                Self::spawn_python_callback_task(
+                    connection_mode.clone(),
+                    reader,
+                    handler.clone(),
+                    ping_handler.clone(),
+                )
+            }),
+            Consumer::Rust(sender) => Some(Self::spawn_rust_streaming_task(
+                connection_mode.clone(),
+                reader,
+                sender.clone(),
+            )),
+        };
+
+        let (writer_tx, writer_rx) = tokio::sync::mpsc::unbounded_channel::<WriterCommand>();
+        let write_task = Self::spawn_write_task(connection_mode.clone(), writer, writer_rx);
 
         // Optionally spawn a heartbeat task to periodically ping server
         let heartbeat_task = heartbeat.as_ref().map(|heartbeat_secs| {
@@ -150,7 +187,7 @@ impl WebSocketClientInner {
                 connection_mode.clone(),
                 *heartbeat_secs,
                 heartbeat_msg.clone(),
-                writer.clone(),
+                writer_tx.clone(),
             )
         });
 
@@ -166,8 +203,9 @@ impl WebSocketClientInner {
         Ok(Self {
             config,
             read_task,
+            write_task,
+            writer_tx,
             heartbeat_task,
-            writer,
             connection_mode,
             reconnect_timeout,
             backoff,
@@ -202,44 +240,42 @@ impl WebSocketClientInner {
         tracing::debug!("Reconnecting");
 
         tokio::time::timeout(self.reconnect_timeout, async {
-            shutdown(
-                self.read_task.take(),
-                self.heartbeat_task.take(),
-                self.writer.clone(),
-            )
-            .await;
-
             let (new_writer, reader) =
                 Self::connect_with_server(&self.config.url, self.config.headers.clone()).await?;
 
-            {
-                let mut guard = self.writer.lock().await;
-                *guard = new_writer;
-                drop(guard);
+            if let Err(e) = self.writer_tx.send(WriterCommand::Update(new_writer)) {
+                tracing::error!("{e}");
             }
 
-            // Spawn new read task
-            if let Some(ref handler) = self.config.handler {
-                self.read_task = Some(Self::spawn_read_task(
-                    reader,
-                    handler.clone(),
-                    self.config.ping_handler.clone(),
-                ));
-            }
+            // Delay before closing connection
+            tokio::time::sleep(Duration::from_millis(100)).await;
 
-            // Optionally spawn new heartbeat task
-            self.heartbeat_task = self.config.heartbeat.as_ref().map(|heartbeat_secs| {
-                Self::spawn_heartbeat_task(
-                    self.connection_mode.clone(),
-                    *heartbeat_secs,
-                    self.config.heartbeat_msg.clone(),
-                    self.writer.clone(),
-                )
-            });
+            if let Some(ref read_task) = self.read_task.take() {
+                if !read_task.is_finished() {
+                    read_task.abort();
+                    tracing::debug!("Aborted task 'read'");
+                }
+            }
 
             self.connection_mode
                 .store(ConnectionMode::Active.as_u8(), Ordering::SeqCst);
 
+            self.read_task = match &self.config.handler {
+                Consumer::Python(handler) => handler.as_ref().map(|handler| {
+                    Self::spawn_python_callback_task(
+                        self.connection_mode.clone(),
+                        reader,
+                        handler.clone(),
+                        self.config.ping_handler.clone(),
+                    )
+                }),
+                Consumer::Rust(sender) => Some(Self::spawn_rust_streaming_task(
+                    self.connection_mode.clone(),
+                    reader,
+                    sender.clone(),
+                )),
+            };
+
             tracing::debug!("Reconnect succeeded");
             Ok(())
         })
@@ -271,17 +307,63 @@ impl WebSocketClientInner {
         }
     }
 
-    fn spawn_read_task(
+    fn spawn_rust_streaming_task(
+        connection_state: Arc<AtomicU8>,
+        mut reader: MessageReader,
+        sender: Sender<Message>,
+    ) -> tokio::task::JoinHandle<()> {
+        tracing::debug!("Started streaming task 'read'");
+
+        let check_interval = Duration::from_millis(10);
+
+        tokio::task::spawn(async move {
+            loop {
+                if !ConnectionMode::from_atomic(&connection_state).is_active() {
+                    break;
+                }
+
+                match tokio::time::timeout(check_interval, reader.next()).await {
+                    Ok(Some(Ok(message))) => {
+                        if let Err(e) = sender.send(message).await {
+                            tracing::error!("Failed to send message: {e}");
+                        }
+                    }
+                    Ok(Some(Err(e))) => {
+                        tracing::error!("Received error message - terminating: {e}");
+                        break;
+                    }
+                    Ok(None) => {
+                        tracing::debug!("No message received - terminating");
+                        break;
+                    }
+                    Err(_) => {
+                        // Timeout - continue loop and check connection mode
+                        continue;
+                    }
+                }
+            }
+        })
+    }
+
+    fn spawn_python_callback_task(
+        connection_state: Arc<AtomicU8>,
         mut reader: MessageReader,
         handler: Arc<PyObject>,
         ping_handler: Option<Arc<PyObject>>,
     ) -> tokio::task::JoinHandle<()> {
         tracing::debug!("Started task 'read'");
 
+        // Interval between checking the connection mode
+        let check_interval = Duration::from_millis(10);
+
         tokio::task::spawn(async move {
             loop {
-                match reader.next().await {
-                    Some(Ok(Message::Binary(data))) => {
+                if !ConnectionMode::from_atomic(&connection_state).is_active() {
+                    break;
+                }
+
+                match tokio::time::timeout(check_interval, reader.next()).await {
+                    Ok(Some(Ok(Message::Binary(data)))) => {
                         tracing::trace!("Received message <binary> {} bytes", data.len());
                         if let Err(e) =
                             Python::with_gil(|py| handler.call1(py, (PyBytes::new(py, &data),)))
@@ -291,7 +373,7 @@ impl WebSocketClientInner {
                         }
                         continue;
                     }
-                    Some(Ok(Message::Text(data))) => {
+                    Ok(Some(Ok(Message::Text(data)))) => {
                         tracing::trace!("Received message: {data}");
                         if let Err(e) = Python::with_gil(|py| {
                             handler.call1(py, (PyBytes::new(py, data.as_bytes()),))
@@ -301,8 +383,8 @@ impl WebSocketClientInner {
                         }
                         continue;
                     }
-                    Some(Ok(Message::Ping(ping))) => {
-                        tracing::trace!("Received ping: {ping:?}",);
+                    Ok(Some(Ok(Message::Ping(ping)))) => {
+                        tracing::trace!("Received ping: {ping:?}");
                         if let Some(ref handler) = ping_handler {
                             if let Err(e) =
                                 Python::with_gil(|py| handler.call1(py, (PyBytes::new(py, &ping),)))
@@ -313,53 +395,141 @@ impl WebSocketClientInner {
                         }
                         continue;
                     }
-                    Some(Ok(Message::Pong(_))) => {
+                    Ok(Some(Ok(Message::Pong(_)))) => {
                         tracing::trace!("Received pong");
                     }
-                    Some(Ok(Message::Close(_))) => {
+                    Ok(Some(Ok(Message::Close(_)))) => {
                         tracing::debug!("Received close message - terminating");
                         break;
                     }
-                    Some(Ok(_)) => (),
-                    Some(Err(e)) => {
+                    Ok(Some(Ok(_))) => (),
+                    Ok(Some(Err(e))) => {
                         tracing::error!("Received error message - terminating: {e}");
                         break;
                     }
                     // Internally tungstenite considers the connection closed when polling
                     // for the next message in the stream returns None.
-                    None => {
+                    Ok(None) => {
                         tracing::debug!("No message received - terminating");
                         break;
                     }
+                    Err(_) => {
+                        // Timeout - continue loop and check connection mode
+                        continue;
+                    }
                 }
             }
         })
     }
 
+    fn spawn_write_task(
+        connection_state: Arc<AtomicU8>,
+        writer: MessageWriter,
+        mut writer_rx: tokio::sync::mpsc::UnboundedReceiver<WriterCommand>,
+    ) -> tokio::task::JoinHandle<()> {
+        tracing::debug!("Started task 'write'");
+
+        // Interval between checking the connection mode
+        let check_interval = Duration::from_millis(10);
+
+        tokio::task::spawn(async move {
+            let mut active_writer = writer;
+
+            loop {
+                match ConnectionMode::from_atomic(&connection_state) {
+                    ConnectionMode::Disconnect => {
+                        // Attempt to close the writer gracefully before exiting,
+                        // we ignore any error as the writer may already be closed.
+                        _ = active_writer.close().await;
+                        break;
+                    }
+                    ConnectionMode::Closed => break,
+                    _ => {}
+                }
+
+                match tokio::time::timeout(check_interval, writer_rx.recv()).await {
+                    Ok(Some(msg)) => {
+                        // Re-check connection mode after receiving a message
+                        let mode = ConnectionMode::from_atomic(&connection_state);
+                        if matches!(mode, ConnectionMode::Disconnect | ConnectionMode::Closed) {
+                            break;
+                        }
+
+                        match msg {
+                            WriterCommand::Update(new_writer) => {
+                                tracing::debug!("Received new writer");
+
+                                // Delay before closing connection
+                                tokio::time::sleep(Duration::from_millis(100)).await;
+
+                                // Attempt to close the writer gracefully on update,
+                                // we ignore any error as the writer may already be closed.
+                                _ = active_writer.close().await;
+
+                                active_writer = new_writer;
+                                tracing::debug!("Updated writer");
+                            }
+                            _ if mode.is_reconnect() => {
+                                tracing::warn!("Skipping message while reconnecting, {msg:?}");
+                                continue;
+                            }
+                            WriterCommand::Send(msg) => {
+                                if let Err(e) = active_writer.send(msg).await {
+                                    tracing::error!("Failed to send message: {e}");
+                                    // Mode is active so trigger reconnection
+                                    tracing::warn!("Writer triggering reconnect");
+                                    connection_state
+                                        .store(ConnectionMode::Reconnect.as_u8(), Ordering::SeqCst);
+                                }
+                            }
+                        }
+                    }
+                    Ok(None) => {
+                        // Channel closed - writer task should terminate
+                        tracing::debug!("Writer channel closed, terminating writer task");
+                        break;
+                    }
+                    Err(_) => {
+                        // Timeout - just continue the loop
+                        continue;
+                    }
+                }
+            }
+
+            // Attempt to close the writer gracefully before exiting,
+            // we ignore any error as the writer may already be closed.
+            _ = active_writer.close().await;
+
+            tracing::debug!("Completed task 'write'");
+        })
+    }
+
     fn spawn_heartbeat_task(
         connection_state: Arc<AtomicU8>,
         heartbeat_secs: u64,
         message: Option<String>,
-        writer: SharedMessageWriter,
+        writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
     ) -> tokio::task::JoinHandle<()> {
         tracing::debug!("Started task 'heartbeat'");
 
         tokio::task::spawn(async move {
             let interval = Duration::from_secs(heartbeat_secs);
+
             loop {
                 tokio::time::sleep(interval).await;
 
                 match ConnectionMode::from_u8(connection_state.load(Ordering::SeqCst)) {
                     ConnectionMode::Active => {
-                        let mut guard = writer.lock().await;
-                        let guard_send_response = match message.clone() {
-                            Some(msg) => guard.send(Message::Text(msg.into())).await,
-                            None => guard.send(Message::Ping(vec![].into())).await,
+                        let msg = match &message {
+                            Some(text) => WriterCommand::Send(Message::Text(text.clone().into())),
+                            None => WriterCommand::Send(Message::Ping(vec![].into())),
                         };
 
-                        match guard_send_response {
-                            Ok(()) => tracing::trace!("Sent ping"),
-                            Err(e) => tracing::error!("Error sending ping: {e}"),
+                        match writer_tx.send(msg) {
+                            Ok(()) => tracing::trace!("Sent heartbeat to writer task"),
+                            Err(e) => {
+                                tracing::error!("Failed to send heartbeat to writer task: {e}");
+                            }
                         }
                     }
                     ConnectionMode::Reconnect => continue,
@@ -372,75 +542,24 @@ impl WebSocketClientInner {
     }
 }
 
-/// Shutdown websocket connection.
-///
-/// Performs orderly WebSocket shutdown:
-/// 1. Sends close frame to server
-/// 2. Waits briefly for frame delivery
-/// 3. Aborts read/heartbeat tasks
-/// 4. Closes underlying connection
-///
-/// This sequence ensures proper protocol compliance and clean resource cleanup.
-async fn shutdown(
-    read_task: Option<tokio::task::JoinHandle<()>>,
-    heartbeat_task: Option<tokio::task::JoinHandle<()>>,
-    writer: SharedMessageWriter,
-) {
-    tracing::debug!("Closing");
-
-    let timeout = Duration::from_secs(5);
-    if tokio::time::timeout(timeout, async {
-        // Send close frame first
-        let mut write_half = writer.lock().await;
-        if let Err(e) = write_half.send(Message::Close(None)).await {
-            // Close frame errors during shutdown are expected
-            tracing::debug!("Error sending close frame: {e}");
-        }
-        drop(write_half);
-
-        tokio::time::sleep(Duration::from_millis(100)).await;
-
-        // Abort tasks
-        if let Some(task) = read_task {
-            if !task.is_finished() {
-                task.abort();
-                tracing::debug!("Aborted read task");
-            }
-        }
-        if let Some(task) = heartbeat_task {
-            if !task.is_finished() {
-                task.abort();
-                tracing::debug!("Aborted heartbeat task");
-            }
-        }
-
-        // Final close of writer
-        let mut write_half = writer.lock().await;
-        if let Err(e) = write_half.close().await {
-            tracing::error!("Error closing writer: {e}");
-        }
-    })
-    .await
-    .is_err()
-    {
-        tracing::error!("Shutdown timed out after {}s", timeout.as_secs());
-    }
-
-    tracing::debug!("Closed");
-}
-
 impl Drop for WebSocketClientInner {
     fn drop(&mut self) {
         if let Some(ref read_task) = self.read_task.take() {
             if !read_task.is_finished() {
                 read_task.abort();
+                tracing::debug!("Aborted task 'read'");
             }
         }
 
-        // Cancel heart beat task
+        if !self.write_task.is_finished() {
+            self.write_task.abort();
+            tracing::debug!("Aborted task 'write'");
+        }
+
         if let Some(ref handle) = self.heartbeat_task.take() {
             if !handle.is_finished() {
                 handle.abort();
+                tracing::debug!("Aborted task 'heartbeat'");
             }
         }
     }
@@ -455,9 +574,9 @@ impl Drop for WebSocketClientInner {
     pyo3::pyclass(module = "nautilus_trader.core.nautilus_pyo3.network")
 )]
 pub struct WebSocketClient {
-    pub(crate) writer: SharedMessageWriter,
     pub(crate) controller_task: tokio::task::JoinHandle<()>,
     pub(crate) connection_mode: Arc<AtomicU8>,
+    pub(crate) writer_tx: tokio::sync::mpsc::UnboundedSender<WriterCommand>,
     pub(crate) rate_limiter: Arc<RateLimiter<String, MonotonicClock>>,
 }
 
@@ -472,18 +591,24 @@ impl WebSocketClient {
         config: WebSocketConfig,
         keyed_quotas: Vec<(String, Quota)>,
         default_quota: Option<Quota>,
+        post_reconnect: Option<Arc<dyn Fn() + Send + Sync>>,
     ) -> Result<(MessageReader, Self), Error> {
+        install_cryptographic_provider();
         let (ws_stream, _) = connect_async(config.url.clone().into_client_request()?).await?;
         let (writer, reader) = ws_stream.split();
-        let writer = Arc::new(Mutex::new(writer));
-
         let inner = WebSocketClientInner::connect_url(config).await?;
+
         let connection_mode = inner.connection_mode.clone();
         let rate_limiter = Arc::new(RateLimiter::new_with_quota(default_quota, keyed_quotas));
+        let writer_tx = inner.writer_tx.clone();
+        if let Err(e) = writer_tx.send(WriterCommand::Update(writer)) {
+            tracing::error!("{e}");
+        }
 
         let controller_task = Self::spawn_controller_task(
             inner,
             connection_mode.clone(),
+            post_reconnect,
             None, // no post_reconnection
             None, // no post_disconnection
         );
@@ -491,9 +616,9 @@ impl WebSocketClient {
         Ok((
             reader,
             Self {
-                writer: writer.clone(),
                 controller_task,
                 connection_mode,
+                writer_tx,
                 rate_limiter,
             },
         ))
@@ -517,14 +642,15 @@ impl WebSocketClient {
     ) -> Result<Self, Error> {
         tracing::debug!("Connecting");
         let inner = WebSocketClientInner::connect_url(config.clone()).await?;
-        let writer = inner.writer.clone();
         let connection_mode = inner.connection_mode.clone();
+        let writer_tx = inner.writer_tx.clone();
 
         let controller_task = Self::spawn_controller_task(
             inner,
             connection_mode.clone(),
-            post_reconnection,
-            post_disconnection,
+            None,               // Rust handler
+            post_reconnection,  // TODO: Deprecated
+            post_disconnection, // TODO: Deprecated
         );
         let rate_limiter = Arc::new(RateLimiter::new_with_quota(default_quota, keyed_quotas));
 
@@ -533,12 +659,12 @@ impl WebSocketClient {
                 Ok(_) => tracing::debug!("Called `post_connection` handler"),
                 Err(e) => tracing::error!("Error calling `post_connection` handler: {e}"),
             });
-        };
+        }
 
         Ok(Self {
-            writer,
             controller_task,
             connection_mode,
+            writer_tx,
             rate_limiter,
         })
     }
@@ -611,7 +737,7 @@ impl WebSocketClient {
 
             if !self.controller_task.is_finished() {
                 self.controller_task.abort();
-                tracing::debug!("Aborted controller task");
+                tracing::debug!("Aborted task 'controller'");
             }
         })
         .await
@@ -628,9 +754,16 @@ impl WebSocketClient {
     /// Sends the given text `data` to the server.
     pub async fn send_text(&self, data: String, keys: Option<Vec<String>>) {
         self.rate_limiter.await_keys_ready(keys).await;
+
+        if !self.is_active() {
+            tracing::error!("Cannot send data - connection not active");
+            return;
+        }
+
         tracing::trace!("Sending text: {data:?}");
-        let mut guard = self.writer.lock().await;
-        if let Err(e) = guard.send(Message::Text(data.into())).await {
+
+        let msg = Message::Text(data.into());
+        if let Err(e) = self.writer_tx.send(WriterCommand::Send(msg)) {
             tracing::error!("Error sending message: {e}");
         }
     }
@@ -638,27 +771,39 @@ impl WebSocketClient {
     /// Sends the given bytes `data` to the server.
     pub async fn send_bytes(&self, data: Vec<u8>, keys: Option<Vec<String>>) {
         self.rate_limiter.await_keys_ready(keys).await;
+
+        if !self.is_active() {
+            tracing::error!("Cannot send data - connection not active");
+            return;
+        }
+
         tracing::trace!("Sending bytes: {data:?}");
-        let mut guard = self.writer.lock().await;
-        if let Err(e) = guard.send(Message::Binary(data.into())).await {
+
+        let msg = Message::Binary(data.into());
+        if let Err(e) = self.writer_tx.send(WriterCommand::Send(msg)) {
             tracing::error!("Error sending message: {e}");
         }
     }
 
     /// Sends a close message to the server.
     pub async fn send_close_message(&self) {
-        let mut guard = self.writer.lock().await;
-        match guard.send(Message::Close(None)).await {
-            Ok(()) => tracing::debug!("Sent close message"),
-            Err(e) => tracing::error!("Error sending close message: {e}"),
+        if !self.is_active() {
+            tracing::error!("Cannot send close message - connection not active");
+            return;
+        }
+
+        let msg = Message::Close(None);
+        if let Err(e) = self.writer_tx.send(WriterCommand::Send(msg)) {
+            tracing::error!("Error sending close message: {e}");
         }
     }
 
     fn spawn_controller_task(
         mut inner: WebSocketClientInner,
         connection_mode: Arc<AtomicU8>,
-        post_reconnection: Option<PyObject>,
-        post_disconnection: Option<PyObject>,
+        post_reconnection: Option<Arc<dyn Fn() + Send + Sync>>,
+        py_post_reconnection: Option<PyObject>, // TODO: Deprecated
+        py_post_disconnection: Option<PyObject>, // TODO: Deprecated
     ) -> tokio::task::JoinHandle<()> {
         tokio::task::spawn(async move {
             tracing::debug!("Started task 'controller'");
@@ -671,14 +816,35 @@ impl WebSocketClient {
 
                 if mode.is_disconnect() {
                     tracing::debug!("Disconnecting");
-                    shutdown(
-                        inner.read_task.take(),
-                        inner.heartbeat_task.take(),
-                        inner.writer.clone(),
-                    )
-                    .await;
 
-                    if let Some(ref handler) = post_disconnection {
+                    let timeout = Duration::from_secs(5);
+                    if tokio::time::timeout(timeout, async {
+                        // Delay awaiting graceful shutdown
+                        tokio::time::sleep(Duration::from_millis(100)).await;
+
+                        if let Some(task) = &inner.read_task {
+                            if !task.is_finished() {
+                                task.abort();
+                                tracing::debug!("Aborted task 'read'");
+                            }
+                        }
+
+                        if let Some(task) = &inner.heartbeat_task {
+                            if !task.is_finished() {
+                                task.abort();
+                                tracing::debug!("Aborted task 'heartbeat'");
+                            }
+                        }
+                    })
+                    .await
+                    .is_err()
+                    {
+                        tracing::error!("Shutdown timed out after {}s", timeout.as_secs());
+                    }
+
+                    tracing::debug!("Closed");
+
+                    if let Some(ref handler) = py_post_disconnection {
                         Python::with_gil(|py| match handler.call0(py) {
                             Ok(_) => tracing::debug!("Called `post_disconnection` handler"),
                             Err(e) => {
@@ -695,9 +861,16 @@ impl WebSocketClient {
                             tracing::debug!("Reconnected successfully");
                             inner.backoff.reset();
 
-                            if let Some(ref handler) = post_reconnection {
+                            if let Some(ref callback) = post_reconnection {
+                                callback();
+                            }
+
+                            // TODO: Python based websocket handlers deprecated (will be removed)
+                            if let Some(ref handler) = py_post_reconnection {
                                 Python::with_gil(|py| match handler.call0(py) {
-                                    Ok(_) => tracing::debug!("Called `post_reconnection` handler"),
+                                    Ok(_) => {
+                                        tracing::debug!("Called `post_reconnection` handler");
+                                    }
                                     Err(e) => tracing::error!(
                                         "Error calling `post_reconnection` handler: {e}"
                                     ),
@@ -718,6 +891,8 @@ impl WebSocketClient {
             inner
                 .connection_mode
                 .store(ConnectionMode::Closed.as_u8(), Ordering::SeqCst);
+
+            tracing::debug!("Completed task 'controller'");
         })
     }
 }
@@ -745,7 +920,7 @@ mod tests {
 
     use crate::{
         ratelimiter::quota::Quota,
-        websocket::{WebSocketClient, WebSocketConfig},
+        websocket::{Consumer, WebSocketClient, WebSocketConfig},
     };
 
     struct TestServer {
@@ -845,7 +1020,7 @@ mod tests {
         let config = WebSocketConfig {
             url: format!("ws://127.0.0.1:{port}"),
             headers: vec![("test".into(), "test".into())],
-            handler: None,
+            handler: Consumer::Python(None),
             heartbeat: None,
             heartbeat_msg: None,
             ping_handler: None,
@@ -889,7 +1064,7 @@ mod tests {
         let config = WebSocketConfig {
             url: "ws://127.0.0.1:9997".into(), // <-- No server
             headers: vec![],
-            handler: None,
+            handler: Consumer::Python(None),
             heartbeat: None,
             heartbeat_msg: None,
             ping_handler: None,
@@ -933,7 +1108,7 @@ mod tests {
         let config = WebSocketConfig {
             url: format!("ws://127.0.0.1:{}", server.port),
             headers: vec![("test".into(), "test".into())],
-            handler: None,
+            handler: Consumer::Python(None),
             heartbeat: None,
             heartbeat_msg: None,
             ping_handler: None,
