@@ -168,15 +168,15 @@ class BetfairExecutionClient(LiveExecutionClient):
         self._update_account_task: asyncio.Task | None = None
 
         # Hot caches:
-        # - filled_qty_cache: Tracks filled quantities separately from order state since Betfair
-        #   only provides cumulative matched sizes (sm). This lets us calculate incremental fills
-        #   while avoiding race conditions with delayed order state updates.
-        # - pending_update_order_client_ids: Tracks orders with pending updates to ensure state
-        #   consistency during asynchronous processing.
-        # - published_executions: Stores published executions per order to avoid duplicates and
-        #   support reconciliation with external systems.
+        # Tracks filled quantities separately from order state since Betfair only provides
+        # cumulative matched sizes (sm). This lets us calculate incremental fills
+        # while avoiding race conditions with delayed order state updates.
         self._filled_qty_cache: dict[ClientOrderId, Quantity] = {}
+
+        # Tracks orders with pending updates to ensure state consistency during asynchronous processing
         self._pending_update_order_client_ids: set[tuple[ClientOrderId, VenueOrderId]] = set()
+
+        # Stores published executions per order to avoid duplicates and support reconciliation
         self._published_executions: dict[ClientOrderId, list[TradeId]] = defaultdict(list)
 
     @property
@@ -208,13 +208,12 @@ class BetfairExecutionClient(LiveExecutionClient):
 
         self._log.debug("Starting 'update_account_state' task")
 
+        # Request one initial update
+        account_state = await self.request_account_state()
+        self._send_account_state(account_state)
+
         if self.config.request_account_state_secs:
             self._update_account_task = self.create_task(self._update_account_state())
-        else:
-            # Request one initial update
-            account_state = await self.request_account_state()
-            self._log.debug(f"Received account state: {account_state}")
-            self._send_account_state(account_state)
 
     async def _reconnect(self) -> None:
         self._log.info("Reconnecting to Betfair")
@@ -227,6 +226,9 @@ class BetfairExecutionClient(LiveExecutionClient):
         await self._client.reconnect()
         await self._stream.reconnect()
 
+        account_state = await self.request_account_state()
+        self._send_account_state(account_state)
+
         if self.config.request_account_state_secs:
             self._update_account_task = self.create_task(self._update_account_state())
 
@@ -267,16 +269,20 @@ class BetfairExecutionClient(LiveExecutionClient):
     async def _update_account_state(self) -> None:
         try:
             while True:
-                self._log.debug("Requesting account state")
-                account_state = await self.request_account_state()
-                self._log.debug(f"Received account state: {account_state}")
-                self._send_account_state(account_state)
-                self._log.debug("Sent account state")
-                await asyncio.sleep(self.config.request_account_state_secs)
+                try:
+                    await asyncio.sleep(self.config.request_account_state_secs)
+                    account_state = await self.request_account_state()
+                    self._send_account_state(account_state)
+                except BetfairError as e:
+                    self._log.warning(str(e))
         except asyncio.CancelledError:
-            self._log.debug("Canceled task '_update_account_state'")
+            self._log.debug("Canceled task 'update_account_state'")
+        except Exception as e:
+            self._log.exception("Error updating account state", e)
 
     async def request_account_state(self) -> AccountState:
+        self._log.debug("Requesting account state")
+
         account_details = await self._client.get_account_details()
         account_funds = await self._client.get_account_funds()
         timestamp = self._clock.timestamp_ns()
@@ -288,6 +294,8 @@ class BetfairExecutionClient(LiveExecutionClient):
             ts_event=timestamp,
             ts_init=timestamp,
         )
+        self._log.debug(f"Received account state: {account_state}")
+
         return account_state
 
     # -- EXECUTION REPORTS ------------------------------------------------------------------------
@@ -951,6 +959,10 @@ class BetfairExecutionClient(LiveExecutionClient):
             trade_id = order_to_trade_id(unmatched_order)
             if trade_id not in self._published_executions[client_order_id]:
                 fill_qty = self._determine_fill_qty(unmatched_order, order)
+                if fill_qty == 0:
+                    self._log.warning(f"Fill size determined as zero for {unmatched_order}")
+                    return
+
                 fill_price = self._determine_fill_price(unmatched_order, order)
                 ts_event = self._get_matched_timestamp(unmatched_order)
 
