@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -14,6 +14,7 @@
 # -------------------------------------------------------------------------------------------------
 
 import asyncio
+from asyncio import TaskGroup
 from decimal import Decimal
 
 import msgspec
@@ -21,6 +22,7 @@ import msgspec
 from nautilus_trader.accounting.accounts.margin import MarginAccount
 from nautilus_trader.adapters.binance.common.enums import BinanceAccountType
 from nautilus_trader.adapters.binance.common.enums import BinanceErrorCode
+from nautilus_trader.adapters.binance.common.enums import BinanceExecutionType
 from nautilus_trader.adapters.binance.config import BinanceExecClientConfig
 from nautilus_trader.adapters.binance.execution import BinanceCommonExecutionClient
 from nautilus_trader.adapters.binance.futures.enums import BinanceFuturesEnumParser
@@ -31,6 +33,7 @@ from nautilus_trader.adapters.binance.futures.http.user import BinanceFuturesUse
 from nautilus_trader.adapters.binance.futures.providers import BinanceFuturesInstrumentProvider
 from nautilus_trader.adapters.binance.futures.schemas.account import BinanceFuturesAccountInfo
 from nautilus_trader.adapters.binance.futures.schemas.account import BinanceFuturesDualSidePosition
+from nautilus_trader.adapters.binance.futures.schemas.account import BinanceFuturesLeverage
 from nautilus_trader.adapters.binance.futures.schemas.account import BinanceFuturesPositionRisk
 from nautilus_trader.adapters.binance.futures.schemas.user import BinanceFuturesAccountUpdateWrapper
 from nautilus_trader.adapters.binance.futures.schemas.user import BinanceFuturesOrderUpdateWrapper
@@ -137,6 +140,12 @@ class BinanceFuturesExecutionClient(BinanceCommonExecutionClient):
             BinanceFuturesEventType.TRADE_LITE: self._handle_trade_lite,
         }
 
+        self._use_trade_lite = config.use_trade_lite
+        if self._use_trade_lite:
+            self._log.info("TRADE_LITE events will be used", LogColor.BLUE)
+
+        self._leverages = config.futures_leverages
+
         # WebSocket futures schema decoders
         self._decoder_futures_user_msg_wrapper = msgspec.json.Decoder(BinanceFuturesUserMsgWrapper)
         self._decoder_futures_order_update_wrapper = msgspec.json.Decoder(
@@ -167,6 +176,16 @@ class BinanceFuturesExecutionClient(BinanceCommonExecutionClient):
         while self.get_account() is None:
             await asyncio.sleep(0.1)
 
+        if self._leverages:
+            async with TaskGroup() as tg:
+                tasks = [
+                    tg.create_task(self._futures_http_account.set_leverage(symbol, leverage))
+                    for symbol, leverage in self._leverages.items()
+                ]
+            for task in tasks:
+                res: BinanceFuturesLeverage = task.result()
+                self._log.info(f"Set default leverage {res.symbol} {res.leverage}X")
+
         account: MarginAccount = self.get_account()
         position_risks = await self._futures_http_account.query_futures_position_risk()
         for position in position_risks:
@@ -263,7 +282,7 @@ class BinanceFuturesExecutionClient(BinanceCommonExecutionClient):
                 client_order_ids=[c.client_order_id.value for c in command.cancels],
             )
         except BinanceError as e:
-            error_code = BinanceErrorCode(e.message["code"])
+            error_code = BinanceErrorCode(int(e.message["code"]))
             if error_code == BinanceErrorCode.CANCEL_REJECTED:
                 self._log.warning(f"Cancel rejected: {e.message}")
             else:
@@ -290,7 +309,8 @@ class BinanceFuturesExecutionClient(BinanceCommonExecutionClient):
 
     def _handle_order_trade_update(self, raw: bytes) -> None:
         order_update = self._decoder_futures_order_update_wrapper.decode(raw)
-        order_update.data.o.handle_order_trade_update(self)
+        if not (self._use_trade_lite and order_update.data.o.x == BinanceExecutionType.TRADE):
+            order_update.data.o.handle_order_trade_update(self)
 
     def _handle_margin_call(self, raw: bytes) -> None:
         self._log.warning("MARGIN CALL received")  # Implement
@@ -303,6 +323,10 @@ class BinanceFuturesExecutionClient(BinanceCommonExecutionClient):
 
     def _handle_trade_lite(self, raw: bytes) -> None:
         trade_lite = self._decoder_futures_trade_lite_wrapper.decode(raw)
-        self._log.warning(
-            f"Received {trade_lite} which is not entirely implemented (needs to be parsed into a fill report)",
-        )
+        if not self._use_trade_lite:
+            self._log.debug(
+                "TradeLite event received but not enabled in config",
+            )
+            return
+        order_data = trade_lite.data.to_order_data()
+        order_data.handle_order_trade_update(self)
