@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -32,6 +32,12 @@ BYBIT_ALL_PRODUCTS: Final[list[BybitProductType]] = [
 # Set of Bybit error codes for which Nautilus will attempt retries,
 # potentially temporary conditions where a retry might make sense.
 BYBIT_RETRY_ERRORS_UTA: Final[set[int]] = {
+    # > ------------------------------------------------------------
+    # > Self defined error codes
+    -10_408,  # Client request timed out
+    # > ------------------------------------------------------------
+    # > Bybit defined error codes
+    # > https://bybit-exchange.github.io/docs/v5/error
     10_000,  # Server Timeout
     10_002,  # The request time exceeds the time window range
     10_006,  # Too many visits. Exceeded the API Rate Limit
