@@ -234,6 +234,7 @@ class BybitExecutionClient(LiveExecutionClient):
                 api_secret=config.api_secret or get_api_secret(config.demo, config.testnet),
                 loop=loop,
                 ws_trade_timeout_secs=config.ws_trade_timeout_secs,
+                recv_window_ms=config.recv_window_ms,
             )
             self._order_single_client = self._ws_order_client
             if config.use_http_batch_api:
@@ -593,7 +594,7 @@ class BybitExecutionClient(LiveExecutionClient):
                     ts_event=millis_to_nanos(ts_event),
                 )
             except Exception as e:
-                self._log.error(f"Failed to generate AccountState: {e}")
+                self._log.exception("Failed to generate AccountState", e)
 
         # Set Leverages
         if self._futures_leverages:
@@ -1093,7 +1094,7 @@ class BybitExecutionClient(LiveExecutionClient):
             else:
                 self._log.error(f"Unknown websocket message topic: {topic}")
         except Exception as e:
-            self._log.error(f"Failed to parse websocket message: {raw.decode()} with error {e}")
+            self._log.exception(f"Failed to parse websocket message: {raw.decode()}", e)
 
     def _handle_account_execution_update(self, raw: bytes) -> None:
         try:
@@ -1126,7 +1127,8 @@ class BybitExecutionClient(LiveExecutionClient):
 
         if client_order_id is None:
             self._log.debug(
-                f"Cannot process order execution for {venue_order_id!r}: no `ClientOrderId` found (most likely due to being an external order)",
+                f"Cannot process order execution for {venue_order_id!r}: "
+                "no `ClientOrderId` found (most likely due to being an external order)",
             )
             return
 
@@ -1147,13 +1149,21 @@ class BybitExecutionClient(LiveExecutionClient):
                 execution.side,
                 trigger_direction,
             )
+            if strategy_id is None:
+                self._log.warning(
+                    f"Cannot process order execution for {client_order_id!r}: "
+                    "no strategy ID found (most likely due to being an external order)",
+                )
+                return
         else:
             strategy_id = order.strategy_id
             order_type = order.order_type
 
         instrument = self._cache.instrument(instrument_id)
         if instrument is None:
-            raise ValueError(f"Cannot handle trade event: instrument {instrument_id} not found")
+            raise ValueError(
+                f"Cannot handle trade event: instrument {instrument_id} not found",
+            )
 
         quote_currency = instrument.quote_currency
         is_maker = execution.isMaker
@@ -1325,13 +1335,13 @@ class BybitExecutionClient(LiveExecutionClient):
                         ts_event=report.ts_last,
                     )
         except Exception as e:
-            self._log.error(repr(e))
+            self._log.exception(repr(e), e)
 
     def _handle_account_wallet_update(self, raw: bytes) -> None:
         try:
             self._process_wallet_update(raw)
         except Exception as e:
-            self._log.exception(f"Failed to handle account wallet update: {e}", e)
+            self._log.exception("Failed to handle account wallet update", e)
 
     def _process_wallet_update(self, raw: bytes) -> None:
         msg: BybitWsAccountWalletMsg = self._decoder_ws_account_wallet_update.decode(raw)
