@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -13,8 +13,10 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
-import asyncio
+from __future__ import annotations
+
 from functools import lru_cache
+from typing import TYPE_CHECKING
 
 from nautilus_trader.adapters.bybit.common.constants import BYBIT_ALL_PRODUCTS
 from nautilus_trader.adapters.bybit.common.credentials import get_api_key
@@ -23,21 +25,27 @@ from nautilus_trader.adapters.bybit.common.enums import BybitProductType
 from nautilus_trader.adapters.bybit.common.urls import get_http_base_url
 from nautilus_trader.adapters.bybit.common.urls import get_ws_base_url_private
 from nautilus_trader.adapters.bybit.common.urls import get_ws_base_url_public
-from nautilus_trader.adapters.bybit.config import BybitDataClientConfig
-from nautilus_trader.adapters.bybit.config import BybitExecClientConfig
+from nautilus_trader.adapters.bybit.common.urls import get_ws_base_url_trade
 from nautilus_trader.adapters.bybit.data import BybitDataClient
 from nautilus_trader.adapters.bybit.execution import BybitExecutionClient
 from nautilus_trader.adapters.bybit.http.client import BybitHttpClient
 from nautilus_trader.adapters.bybit.providers import BybitInstrumentProvider
-from nautilus_trader.cache.cache import Cache
-from nautilus_trader.common.component import LiveClock
-from nautilus_trader.common.component import MessageBus
-from nautilus_trader.config import InstrumentProviderConfig
 from nautilus_trader.core.nautilus_pyo3 import Quota
 from nautilus_trader.live.factories import LiveDataClientFactory
 from nautilus_trader.live.factories import LiveExecClientFactory
 
 
+if TYPE_CHECKING:
+    import asyncio
+
+    from nautilus_trader.adapters.bybit.config import BybitDataClientConfig
+    from nautilus_trader.adapters.bybit.config import BybitExecClientConfig
+    from nautilus_trader.cache.cache import Cache
+    from nautilus_trader.common.component import LiveClock
+    from nautilus_trader.common.component import MessageBus
+    from nautilus_trader.config import InstrumentProviderConfig
+
+
 @lru_cache(1)
 def get_bybit_http_client(
     clock: LiveClock,
@@ -46,6 +54,7 @@ def get_bybit_http_client(
     base_url: str | None = None,
     is_demo: bool = False,
     is_testnet: bool = False,
+    recv_window_ms: int = 5_000,
 ) -> BybitHttpClient:
     """
     Cache and return a Bybit HTTP client with the given key and secret.
@@ -66,6 +75,8 @@ def get_bybit_http_client(
         If the client is connecting to the demo API.
     is_testnet : bool, default False
         If the client is connecting to the testnet API.
+    recv_window_ms : PositiveInt, default 5000
+        The receive window (milliseconds) for Bybit HTTP requests.
 
     Returns
     -------
@@ -88,6 +99,7 @@ def get_bybit_http_client(
         api_key=key,
         api_secret=secret,
         base_url=http_base_url,
+        recv_window_ms=recv_window_ms,
         ratelimiter_quotas=ratelimiter_quotas,
         ratelimiter_default_quota=ratelimiter_default_quota,
     )
@@ -176,6 +188,7 @@ class BybitLiveDataClientFactory(LiveDataClientFactory):
             base_url=config.base_url_http,
             is_demo=config.demo,
             is_testnet=config.testnet,
+            recv_window_ms=config.recv_window_ms,
         )
         provider = get_bybit_instrument_provider(
             client=client,
@@ -248,6 +261,7 @@ class BybitLiveExecClientFactory(LiveExecClientFactory):
             base_url=config.base_url_http,
             is_demo=config.demo,
             is_testnet=config.testnet,
+            recv_window_ms=config.recv_window_ms,
         )
         provider = get_bybit_instrument_provider(
             client=client,
@@ -255,7 +269,10 @@ class BybitLiveExecClientFactory(LiveExecClientFactory):
             product_types=frozenset(config.product_types or BYBIT_ALL_PRODUCTS),
             config=config.instrument_provider,
         )
-        base_url_ws: str = get_ws_base_url_private(config.testnet)
+
+        base_url_ws_private: str = get_ws_base_url_private(config.testnet)
+        base_url_ws_trade: str = get_ws_base_url_trade(config.testnet)
+
         return BybitExecutionClient(
             loop=loop,
             client=client,
@@ -264,7 +281,8 @@ class BybitLiveExecClientFactory(LiveExecClientFactory):
             clock=clock,
             instrument_provider=provider,
             product_types=config.product_types or [BybitProductType.SPOT],
-            base_url_ws=config.base_url_ws or base_url_ws,
+            base_url_ws_private=config.base_url_ws_private or base_url_ws_private,
+            base_url_ws_trade=config.base_url_ws_trade or base_url_ws_trade,
             config=config,
             name=name,
         )
