@@ -55,11 +55,18 @@ class DatabentoDataLoader:
 
     """
 
-    def __init__(self) -> None:
+    def __init__(
+        self,
+        venue_dataset_map: dict[str, str] | None = None,
+    ) -> None:
         self._pyo3_loader: nautilus_pyo3.DatabentoDataLoader = nautilus_pyo3.DatabentoDataLoader(
             str(PUBLISHERS_FILEPATH),
         )
 
+        if venue_dataset_map:
+            for venue, dataset in venue_dataset_map.items():
+                self._pyo3_loader.set_dataset_for_venue(dataset, nautilus_pyo3.Venue(venue))
+
     def load_publishers(self, path: PathLike[str] | str) -> None:
         """
         Load publisher details from the JSON file at the given path.
@@ -103,13 +110,10 @@ class DatabentoDataLoader:
 
         """
         dataset = self._pyo3_loader.get_dataset_for_venue(nautilus_pyo3.Venue(venue.value))
+
         if dataset is None:
             raise ValueError(f"No Databento dataset for venue '{venue}'")
 
-        # TODO: Temporary workaround
-        if dataset == "XNAS.BASIC":
-            return "XNAS.ITCH"
-
         return dataset
 
     def from_dbn_file(  # noqa: C901 (too complex)
@@ -171,16 +175,18 @@ class DatabentoDataLoader:
             if instrument_id is not None
             else None
         )
-
         schema = self._pyo3_loader.schema_for_file(str(path))
+
         if schema is None:
             raise RuntimeError("Loading files with mixed schemas not currently supported")
 
         match schema:
             case DatabentoSchema.DEFINITION.value:
                 data = self._pyo3_loader.load_instruments(str(path), use_exchange_as_venue)
+
                 if as_legacy_cython:
                     data = instruments_from_pyo3(data)
+
                 return data
             case DatabentoSchema.MBO.value:
                 if as_legacy_cython:
@@ -193,6 +199,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     if include_trades:
@@ -200,6 +207,7 @@ class DatabentoDataLoader:
                             "Cannot load `OrderBookDelta` and `Trade` objects together, "
                             "set `include_trades` to False",
                         )
+
                     return self._pyo3_loader.load_order_book_deltas(
                         filepath=str(path),
                         instrument_id=pyo3_instrument_id,
@@ -216,6 +224,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     if include_trades:
@@ -223,6 +232,7 @@ class DatabentoDataLoader:
                             "Cannot load `QuoteTick` and `TradeTick` objects together, "
                             "set `include_trades` to False",
                         )
+
                     return self._pyo3_loader.load_quotes(
                         filepath=str(path),
                         instrument_id=pyo3_instrument_id,
@@ -238,6 +248,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     return self._pyo3_loader.load_bbo_quotes(
@@ -255,6 +266,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     return self._pyo3_loader.load_order_book_depth10(str(path), pyo3_instrument_id)
@@ -268,6 +280,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     return self._pyo3_loader.load_trades(str(path), pyo3_instrument_id)
@@ -287,6 +300,7 @@ class DatabentoDataLoader:
                     data = capsule_to_list(capsule)
                     # Drop encapsulated `CVec` as data is now transferred
                     drop_cvec_pycapsule(capsule)
+
                     return data
                 else:
                     return self._pyo3_loader.load_bars(
@@ -301,12 +315,14 @@ class DatabentoDataLoader:
                 )
                 if as_legacy_cython:
                     return InstrumentStatus.from_pyo3_list(data)
+
                 return data
             case DatabentoSchema.IMBALANCE.value:
                 if as_legacy_cython:
                     raise ValueError(
                         "Cannot load `DatabentoImbalance` as Cython objects, set `as_legacy_cython` to False",
                     )
+
                 return self._pyo3_loader.load_imbalance(
                     filepath=str(path),
                     instrument_id=pyo3_instrument_id,
@@ -317,6 +333,7 @@ class DatabentoDataLoader:
                     raise ValueError(
                         "Cannot load `DatabentoStatistics` as Cython objects, set `as_legacy_cython` to False",
                     )
+
                 return self._pyo3_loader.load_statistics(
                     filepath=str(path),
                     instrument_id=pyo3_instrument_id,
