@@ -19,6 +19,7 @@ Define the base class for dYdX endpoints.
 from typing import Any
 
 import msgspec
+from msgspec import DecodeError
 
 from nautilus_trader.adapters.dydx.common.enums import DYDXEndpointType
 from nautilus_trader.adapters.dydx.http.client import DYDXHttpClient
@@ -64,7 +65,7 @@ class DYDXHttpEndpoint:
             max_retries=5,
             retry_delay_secs=1.0,
             logger=Logger(name="DYDXHttpEndpoint"),
-            exc_types=(HttpTimeoutError, HttpError, DYDXError),
+            exc_types=(HttpTimeoutError, HttpError, DYDXError, DecodeError),
             retry_check=should_retry,
         )
 
