@@ -371,7 +371,7 @@ class OKXWebsocketClient:
         self._is_running = False
 
         if self._client is None:
-            self._log.warning("Cannot disconnect: not connected.")
+            self._log.warning("Cannot disconnect: not connected")
             return
 
         await self._client.disconnect()
