@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -108,7 +108,7 @@ class PolymarketWebSocketClient:
         bool
 
         """
-        return self._client is not None and self._client.is_alive()
+        return self._client is not None and self._client.is_active()
 
     def is_disconnected(self) -> bool:
         """
