@@ -13,6 +13,7 @@
 #  limitations under the License.
 # -------------------------------------------------------------------------------------------------
 
+import json
 from decimal import Decimal
 
 import pandas as pd
@@ -29,6 +30,7 @@ from nautilus_trader.common.component import init_logging
 from nautilus_trader.common.component import is_logging_initialized
 from nautilus_trader.common.config import ActorFactory
 from nautilus_trader.common.config import InvalidConfiguration
+from nautilus_trader.common.enums import LogColor
 from nautilus_trader.core import nautilus_pyo3
 from nautilus_trader.core.correctness import PyCondition
 from nautilus_trader.core.datetime import dt_to_unix_nanos
@@ -38,6 +40,7 @@ from nautilus_trader.core.inspect import is_nautilus_class
 from nautilus_trader.core.nautilus_pyo3 import DataBackendSession
 from nautilus_trader.model import BOOK_DATA_TYPES
 from nautilus_trader.model.data import Bar
+from nautilus_trader.model.data import BarType
 from nautilus_trader.model.data import capsule_to_list
 from nautilus_trader.model.enums import AccountType
 from nautilus_trader.model.enums import BookType
@@ -70,14 +73,13 @@ class BacktestNode:
 
     """
 
-    def __init__(self, configs: list[BacktestRunConfig]):
+    def __init__(self, configs: list[BacktestRunConfig]) -> None:
         PyCondition.not_none(configs, "configs")
         PyCondition.not_empty(configs, "configs")
         PyCondition.is_true(
             all(isinstance(config, BacktestRunConfig) for config in configs),
             "configs",
         )
-
         self._validate_configs(configs)
 
         self._configs: list[BacktestRunConfig] = configs
@@ -136,59 +138,22 @@ class BacktestNode:
         """
         return list(self._engines.values())
 
-    def run(self, raise_exception=False) -> list[BacktestResult]:
-        """
-        Run the backtest node which will synchronously execute the list of loaded
-        backtest run configs.
-
-        Parameters
-        ----------
-        raise_exception : bool, default False
-            If True, an exception raised from a backtest will be re-raised and halt the node.
-            If False, exceptions raised from backtest(s) will be printed to stdout.
-
-        Returns
-        -------
-        list[BacktestResult]
-            The results of the backtest runs.
-
-        """
-        results: list[BacktestResult] = []
-        for config in self._configs:
-            try:
-                result = self._run(
-                    run_config_id=config.id,
-                    engine_config=config.engine,
-                    venue_configs=config.venues,
-                    data_configs=config.data,
-                    chunk_size=config.chunk_size,
-                    dispose_on_completion=config.dispose_on_completion,
-                    start=config.start,
-                    end=config.end,
-                )
-                results.append(result)
-            except Exception as e:
-                # Broad catch all prevents a single backtest run from halting
-                # the execution of the other backtests (such as a zero balance exception).
-                if not is_logging_initialized():
-                    _guard = init_logging()
-                log = Logger(type(self).__name__)
-                log.error(f"Error running backtest: {e}")
-                log.info(f"Config: {config}")
-
-                if raise_exception:
-                    raise e
-
-        return results
+    def dispose(self):
+        for engine in self.get_engines():
+            if not engine.trader.is_disposed:
+                engine.dispose()
 
     def _validate_configs(self, configs: list[BacktestRunConfig]) -> None:  # noqa: C901
         venue_ids: list[Venue] = []
+
         for config in configs:
             venue_ids += [Venue(c.name) for c in config.venues]
 
         for config in configs:
             for data_config in config.data:
-                if data_config.instrument_id is None:
+                used_instrument_ids: list[InstrumentId] = get_instrument_ids(data_config)
+
+                if len(used_instrument_ids) == 0:
                     continue  # No instrument associated with data
 
                 if data_config.start_time is not None and data_config.end_time is not None:
@@ -200,12 +165,12 @@ class BacktestNode:
                             f"`end_time` ({data_config.end_time}) is before `start_time` ({data_config.start_time})",
                         )
 
-                instrument_id: InstrumentId = data_config.instrument_id
-                if instrument_id.venue not in venue_ids:
-                    raise InvalidConfiguration(
-                        f"Venue '{instrument_id.venue}' for {instrument_id} "
-                        f"does not have a `BacktestVenueConfig`",
-                    )
+                for instrument_id in used_instrument_ids:
+                    if instrument_id.venue not in venue_ids:
+                        raise InvalidConfiguration(
+                            f"Venue '{instrument_id.venue}' for {instrument_id} "
+                            f"does not have a `BacktestVenueConfig`",
+                        )
 
             for venue_config in config.venues:
                 venue = Venue(venue_config.name)
@@ -225,82 +190,60 @@ class BacktestNode:
                             f"No order book data available for {venue} with book type {venue_config.book_type}",
                         )
 
-    def _create_engine(
-        self,
-        run_config_id: str,
-        config: BacktestEngineConfig,
-        venue_configs: list[BacktestVenueConfig],
-        data_configs: list[BacktestDataConfig],
-    ) -> BacktestEngine:
-        # Build the backtest engine
-        engine = BacktestEngine(config=config)
-        self._engines[run_config_id] = engine
+    def run(self, raise_exception=False) -> list[BacktestResult]:
+        """
+        Run the backtest node which will synchronously execute the list of loaded
+        backtest run configs.
 
-        # Assign the global logging system guard to keep it alive for
-        # the duration of the nodes runs.
-        log_guard = engine.kernel.get_log_guard()
-        if log_guard:
-            self._log_guard = log_guard
+        Parameters
+        ----------
+        raise_exception : bool, default False
+            If True, an exception raised from a backtest will be re-raised and halt the node.
+            If False, exceptions raised from backtest(s) will be printed to stdout.
 
-        # Add venues (must be added prior to instruments)
-        for config in venue_configs:
-            base_currency: str | None = config.base_currency
-            leverages = (
-                {InstrumentId.from_str(i): Decimal(v) for i, v in config.leverages.items()}
-                if config.leverages
-                else {}
-            )
+        Returns
+        -------
+        list[BacktestResult]
+            The results of the backtest runs.
 
-            engine.add_venue(
-                venue=Venue(config.name),
-                oms_type=OmsType[config.oms_type],
-                account_type=AccountType[config.account_type],
-                base_currency=Currency.from_str(base_currency) if base_currency else None,
-                starting_balances=[Money.from_str(m) for m in config.starting_balances],
-                default_leverage=Decimal(config.default_leverage),
-                leverages=leverages,
-                book_type=book_type_from_str(config.book_type),
-                routing=config.routing,
-                modules=[ActorFactory.create(module) for module in (config.modules or [])],
-                frozen_account=config.frozen_account,
-                reject_stop_orders=config.reject_stop_orders,
-                support_gtd_orders=config.support_gtd_orders,
-                support_contingent_orders=config.support_contingent_orders,
-                use_position_ids=config.use_position_ids,
-                use_random_ids=config.use_random_ids,
-                use_reduce_only=config.use_reduce_only,
-                bar_execution=config.bar_execution,
-                bar_adaptive_high_low_ordering=config.bar_adaptive_high_low_ordering,
-                trade_execution=config.trade_execution,
-            )
+        """
+        results: list[BacktestResult] = []
 
-        # Add instruments
-        for config in data_configs:
-            if is_nautilus_class(config.data_type):
-                catalog = self.load_catalog(config)
-                instruments = catalog.instruments(instrument_ids=config.instrument_id)
-                for instrument in instruments or []:
-                    if instrument.id not in engine.cache.instrument_ids():
-                        engine.add_instrument(instrument)
-
-        return engine
-
-    def _load_engine_data(self, engine: BacktestEngine, result: CatalogDataResult) -> None:
-        if is_nautilus_class(result.data_cls):
-            engine.add_data(
-                data=result.data,
-                sort=True,  # Already sorted from backend
-            )
-        else:
-            if not result.client_id:
-                raise ValueError(
-                    f"Data type {result.data_cls} not setup for loading into `BacktestEngine`",
+        for config in self._configs:
+            try:
+                result = self._run(
+                    run_config_id=config.id,
+                    engine_config=config.engine,
+                    venue_configs=config.venues,
+                    data_configs=config.data,
+                    chunk_size=config.chunk_size,
+                    dispose_on_completion=config.dispose_on_completion,
+                    start=config.start,
+                    end=config.end,
                 )
-            engine.add_data(
-                data=result.data,
-                client_id=result.client_id,
-                sort=True,  # Already sorted from backend
-            )
+                results.append(result)
+            except Exception as e:
+                # Broad catch all prevents a single backtest run from halting
+                # the execution of the other backtests (such as a zero balance exception).
+                if not is_logging_initialized():
+                    _guard = init_logging()
+
+                log = Logger(type(self).__name__)
+                log.exception("Error running backtest", e)
+                if config.engine is not None:
+                    log.info("Engine config:", LogColor.MAGENTA)
+                    log.info(json.dumps(json.loads(config.engine.json()), indent=2))
+                log.info("Venue configs:", LogColor.MAGENTA)
+                for venue_config in config.venues:
+                    log.info(json.dumps(json.loads(venue_config.json()), indent=2))
+                log.info("Data configs:", LogColor.MAGENTA)
+                for data_config in config.data:
+                    log.info(json.dumps(json.loads(data_config.json()), indent=2))
+
+                if raise_exception:
+                    raise e
+
+        return results
 
     def _run(
         self,
@@ -348,7 +291,67 @@ class BacktestNode:
 
         return engine.get_result()
 
-    def _run_streaming(
+    def _create_engine(
+        self,
+        run_config_id: str,
+        config: BacktestEngineConfig,
+        venue_configs: list[BacktestVenueConfig],
+        data_configs: list[BacktestDataConfig],
+    ) -> BacktestEngine:
+        # Build the backtest engine
+        engine = BacktestEngine(config=config)
+        self._engines[run_config_id] = engine
+
+        # Assign the global logging system guard to keep it alive for
+        # the duration of the nodes runs.
+        log_guard = engine.kernel.get_log_guard()
+
+        if log_guard:
+            self._log_guard = log_guard
+
+        # Add venues (must be added prior to instruments)
+        for config in venue_configs:
+            engine.add_venue(
+                venue=Venue(config.name),
+                oms_type=OmsType[config.oms_type],
+                account_type=AccountType[config.account_type],
+                base_currency=get_base_currency(config),
+                starting_balances=get_starting_balances(config),
+                default_leverage=Decimal(config.default_leverage),
+                leverages=get_leverages(config),
+                book_type=book_type_from_str(config.book_type),
+                routing=config.routing,
+                modules=[ActorFactory.create(module) for module in (config.modules or [])],
+                frozen_account=config.frozen_account,
+                reject_stop_orders=config.reject_stop_orders,
+                support_gtd_orders=config.support_gtd_orders,
+                support_contingent_orders=config.support_contingent_orders,
+                use_position_ids=config.use_position_ids,
+                use_random_ids=config.use_random_ids,
+                use_reduce_only=config.use_reduce_only,
+                bar_execution=config.bar_execution,
+                bar_adaptive_high_low_ordering=config.bar_adaptive_high_low_ordering,
+                trade_execution=config.trade_execution,
+            )
+
+        # Add instruments
+        for config in data_configs:
+            if is_nautilus_class(config.data_type):
+                catalog = self.load_catalog(config)
+                used_instrument_ids = get_instrument_ids(config)
+
+                # None to query all instruments
+                instruments = catalog.instruments(
+                    instrument_ids=(used_instrument_ids if len(used_instrument_ids) > 0 else None),
+                )
+
+                for instrument in instruments or []:
+                    if instrument.id not in engine.cache.instrument_ids():
+                        engine.add_instrument(instrument)
+
+        return engine
+
+    def _run_streaming(  # noqa: C901
         self,
         run_config_id: str,
         engine: BacktestEngine,
@@ -363,30 +366,38 @@ class BacktestNode:
         # Add query for all data configs
         for config in data_configs:
             catalog = self.load_catalog(config)
-            if config.data_type == Bar:
-                # TODO: Temporary hack - improve bars config and decide implementation with `filter_expr`
-                assert config.instrument_id, "No `instrument_id` for Bar data config"
-                assert config.bar_spec, "No `bar_spec` for Bar data config"
-                bar_type = f"{config.instrument_id}-{config.bar_spec}-EXTERNAL"
-            else:
-                bar_type = None
-
             used_start = config.start_time
+            used_end = config.end_time
+
             if used_start is not None or start is not None:
                 result = max_date(used_start, start)
                 used_start = result.isoformat() if result else None
 
-            used_end = config.end_time
             if used_end is not None or end is not None:
                 result = min_date(used_end, end)
                 used_end = result.isoformat() if result else None
 
+            used_instrument_ids = get_instrument_ids(config)
+            used_bar_types = []
+
+            if config.data_type == Bar:
+                if config.bar_types is None and config.instrument_ids is None:
+                    assert config.instrument_id, "No `instrument_id` for Bar data config"
+                    assert config.bar_spec, "No `bar_spec` for Bar data config"
+
+                if config.instrument_id is not None and config.bar_spec is not None:
+                    bar_type = f"{config.instrument_id}-{config.bar_spec}-EXTERNAL"
+                    used_bar_types = [bar_type]
+                elif config.bar_types is not None:
+                    used_bar_types = config.bar_types
+                elif config.instrument_ids is not None and config.bar_spec is not None:
+                    for instrument_id in config.instrument_ids:
+                        used_bar_types.append(f"{instrument_id}-{config.bar_spec}-EXTERNAL")
+
             session = catalog.backend_session(
                 data_cls=config.data_type,
-                instrument_ids=(
-                    [config.instrument_id] if config.instrument_id and not bar_type else []
-                ),
-                bar_types=[bar_type] if bar_type else [],
+                instrument_ids=used_instrument_ids,
+                bar_types=(used_bar_types if len(used_bar_types) > 0 else None),
                 start=used_start,
                 end=used_end,
                 session=session,
@@ -420,15 +431,18 @@ class BacktestNode:
         # Load data
         for config in data_configs:
             t0 = pd.Timestamp.now()
+            used_instrument_ids = get_instrument_ids(config)
             engine.logger.info(
-                f"Reading {config.data_type} data for instrument={config.instrument_id}.",
+                f"Reading {config.data_type} data for instrument_ids={used_instrument_ids}.",
             )
             result: CatalogDataResult = self.load_data_config(config, start, end)
-            if config.instrument_id and result.instrument is None:
+
+            if len(used_instrument_ids) > 0 and result.instruments is None:
                 engine.logger.warning(
-                    f"Requested instrument_id={result.instrument} from data_config not found in catalog",
+                    f"Requested instrument_ids={used_instrument_ids} from data_config not found in catalog",
                 )
                 continue
+
             if not result.data:
                 engine.logger.warning(f"No data found for {config}")
                 continue
@@ -437,20 +451,14 @@ class BacktestNode:
             engine.logger.info(
                 f"Read {len(result.data):,} events from parquet in {pd.Timedelta(t1 - t0)}s",
             )
+
             self._load_engine_data(engine=engine, result=result)
+
             t2 = pd.Timestamp.now()
             engine.logger.info(f"Engine load took {pd.Timedelta(t2 - t1)}s")
 
         engine.run(start=start, end=end, run_config_id=run_config_id)
 
-    @classmethod
-    def load_catalog(cls, config: BacktestDataConfig) -> ParquetDataCatalog:
-        return ParquetDataCatalog(
-            path=config.catalog_path,
-            fs_protocol=config.catalog_fs_protocol,
-            fs_storage_options=config.catalog_fs_storage_options,
-        )
-
     @classmethod
     def load_data_config(
         cls,
@@ -459,13 +467,15 @@ class BacktestNode:
         end: str | int | None = None,
     ) -> CatalogDataResult:
         catalog: ParquetDataCatalog = cls.load_catalog(config)
+        used_instrument_ids = get_instrument_ids(config)
 
         instruments = (
-            catalog.instruments(instrument_ids=[config.instrument_id])
-            if config.instrument_id
+            catalog.instruments(instrument_ids=used_instrument_ids)
+            if len(used_instrument_ids) > 0
             else None
         )
-        if config.instrument_id and not instruments:
+
+        if len(used_instrument_ids) > 0 and not instruments:
             return CatalogDataResult(data_cls=config.data_type, data=[])
 
         config_query = config.query
@@ -481,11 +491,80 @@ class BacktestNode:
         return CatalogDataResult(
             data_cls=config.data_type,
             data=catalog.query(**config_query),
-            instrument=instruments[0] if instruments else None,
+            instruments=instruments,
             client_id=ClientId(config.client_id) if config.client_id else None,
         )
 
-    def dispose(self):
-        for engine in self.get_engines():
-            if not engine.trader.is_disposed:
-                engine.dispose()
+    @classmethod
+    def load_catalog(cls, config: BacktestDataConfig) -> ParquetDataCatalog:
+        return ParquetDataCatalog(
+            path=config.catalog_path,
+            fs_protocol=config.catalog_fs_protocol,
+            fs_storage_options=config.catalog_fs_storage_options,
+        )
+
+    def _load_engine_data(self, engine: BacktestEngine, result: CatalogDataResult) -> None:
+        if is_nautilus_class(result.data_cls):
+            engine.add_data(
+                data=result.data,
+                sort=True,  # Already sorted from backend
+            )
+        else:
+            if not result.client_id:
+                raise ValueError(
+                    f"Data type {result.data_cls} not setup for loading into `BacktestEngine`",
+                )
+
+            engine.add_data(
+                data=result.data,
+                client_id=result.client_id,
+                sort=True,  # Already sorted from backend
+            )
+
+
+def get_instrument_ids(config: BacktestDataConfig) -> list[InstrumentId]:
+    instrument_ids = []
+
+    if config.instrument_id:
+        instrument_id = (
+            InstrumentId.from_str(config.instrument_id)
+            if type(config.instrument_id) is str
+            else config.instrument_id
+        )
+        instrument_ids = [instrument_id]
+    elif config.instrument_ids:
+        instrument_ids = [
+            (InstrumentId.from_str(instrument_id) if type(instrument_id) is str else instrument_id)
+            for instrument_id in config.instrument_ids
+        ]
+    elif config.bar_types:
+        bar_types: list[BarType] = [
+            BarType.from_str(bar_type) if type(bar_type) is str else bar_type
+            for bar_type in config.bar_types
+        ]
+        instrument_ids = [bar_type.instrument_id for bar_type in bar_types]
+
+    return instrument_ids
+
+
+def get_starting_balances(config: BacktestVenueConfig) -> list[Money]:
+    starting_balances = []
+
+    for balance in config.starting_balances:
+        starting_balances.append(Money.from_str(balance) if type(balance) is str else balance)
+
+    return starting_balances
+
+
+def get_base_currency(config: BacktestVenueConfig) -> Currency | None:
+    base_currency = config.base_currency
+
+    return Currency.from_str(base_currency) if type(base_currency) is str else base_currency
+
+
+def get_leverages(config: BacktestVenueConfig) -> dict[InstrumentId, Decimal]:
+    return (
+        {InstrumentId.from_str(i): Decimal(v) for i, v in config.leverages.items()}
+        if config.leverages
+        else {}
+    )
