@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -65,6 +65,7 @@ from nautilus_trader.live.config import RoutingConfig
 from nautilus_trader.live.config import TradingNodeConfig
 from nautilus_trader.persistence.config import DataCatalogConfig
 from nautilus_trader.persistence.config import StreamingConfig
+from nautilus_trader.portfolio.config import PortfolioConfig
 from nautilus_trader.risk.config import RiskEngineConfig
 from nautilus_trader.system.config import NautilusKernelConfig
 from nautilus_trader.trading.config import ImportableStrategyConfig
@@ -108,6 +109,7 @@ __all__ = [
     "NonNegativeFloat",
     "NonNegativeInt",
     "OrderEmulatorConfig",
+    "PortfolioConfig",
     "PositiveFloat",
     "PositiveInt",
     "RiskEngineConfig",
