@@ -15,7 +15,6 @@
 
 import asyncio
 import math
-import traceback
 import uuid
 from asyncio import Queue
 from collections import Counter
@@ -148,6 +147,9 @@ class LiveExecutionEngine(ExecutionEngine):
         self._inflight_check_task: asyncio.Task | None = None
         self._own_books_audit_task: asyncio.Task | None = None
         self._open_check_task: asyncio.Task | None = None
+        self._purge_closed_orders_task: asyncio.Task | None = None
+        self._purge_closed_positions_task: asyncio.Task | None = None
+        self._purge_account_events_task: asyncio.Task | None = None
         self._kill: bool = False
 
         # Configuration
@@ -162,6 +164,12 @@ class LiveExecutionEngine(ExecutionEngine):
         self.own_books_audit_interval_secs: float | None = config.own_books_audit_interval_secs
         self.open_check_interval_secs: float | None = config.open_check_interval_secs
         self.open_check_open_only: float | None = config.open_check_open_only
+        self.purge_closed_orders_interval_mins = config.purge_closed_orders_interval_mins
+        self.purge_closed_orders_buffer_mins = config.purge_closed_orders_buffer_mins
+        self.purge_closed_positions_interval_mins = config.purge_closed_positions_interval_mins
+        self.purge_closed_positions_buffer_mins = config.purge_closed_positions_buffer_mins
+        self.purge_account_events_interval_mins = config.purge_account_events_interval_mins
+        self.purge_account_events_lookback_mins = config.purge_account_events_lookback_mins
         self._inflight_check_threshold_ns: int = millis_to_nanos(self.inflight_check_threshold_ms)
 
         self._log.info(f"{config.reconciliation=}", LogColor.BLUE)
@@ -174,6 +182,12 @@ class LiveExecutionEngine(ExecutionEngine):
         self._log.info(f"{config.own_books_audit_interval_secs=}", LogColor.BLUE)
         self._log.info(f"{config.open_check_interval_secs=}", LogColor.BLUE)
         self._log.info(f"{config.open_check_open_only=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_closed_orders_interval_mins=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_closed_orders_buffer_mins=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_closed_positions_interval_mins=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_closed_positions_buffer_mins=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_account_events_interval_mins=}", LogColor.BLUE)
+        self._log.info(f"{config.purge_account_events_lookback_mins=}", LogColor.BLUE)
 
         # Register endpoints
         self._msgbus.register(endpoint="ExecEngine.reconcile_report", handler=self.reconcile_report)
@@ -382,6 +396,24 @@ class LiveExecutionEngine(ExecutionEngine):
                 name="open_check",
             )
 
+        if self.purge_closed_orders_interval_mins and not self._purge_closed_orders_task:
+            self._purge_closed_orders_task = self._loop.create_task(
+                self._purge_closed_orders_loop(self.purge_closed_orders_interval_mins),
+                name="purge_closed_orders",
+            )
+
+        if self.purge_closed_positions_interval_mins and not self._purge_closed_positions_task:
+            self._purge_closed_positions_task = self._loop.create_task(
+                self._purge_closed_positions_loop(self.purge_closed_positions_interval_mins),
+                name="purge_closed_positions",
+            )
+
+        if self.purge_account_events_interval_mins and not self._purge_account_events_task:
+            self._purge_account_events_task = self._loop.create_task(
+                self._purge_account_events_loop(self.purge_account_events_interval_mins),
+                name="purge_account_events",
+            )
+
     def _on_stop(self) -> None:
         if self._inflight_check_task:
             self._log.debug(f"Canceling task '{self._inflight_check_task.get_name()}'")
@@ -398,6 +430,21 @@ class LiveExecutionEngine(ExecutionEngine):
             self._open_check_task.cancel()
             self._open_check_task = None
 
+        if self._purge_closed_orders_task:
+            self._log.debug(f"Canceling task '{self._purge_closed_orders_task.get_name()}'")
+            self._purge_closed_orders_task.cancel()
+            self._purge_closed_orders_task = None
+
+        if self._purge_closed_positions_task:
+            self._log.debug(f"Canceling task '{self._purge_closed_positions_task.get_name()}'")
+            self._purge_closed_positions_task.cancel()
+            self._purge_closed_positions_task = None
+
+        if self._purge_account_events_task:
+            self._log.debug(f"Canceling task '{self._purge_account_events_task.get_name()}'")
+            self._purge_account_events_task.cancel()
+            self._purge_account_events_task = None
+
         if self._kill:
             return  # Avoids enqueuing unnecessary sentinel messages when termination already signaled
 
@@ -417,7 +464,7 @@ class LiveExecutionEngine(ExecutionEngine):
         except asyncio.CancelledError:
             self._log.warning("Canceled task 'run_cmd_queue'")
         except Exception as e:
-            self._log.error(f"{e!r}\n{traceback.format_exc()}")
+            self._log.exception(f"{e!r}", e)
         finally:
             stopped_msg = "Command message queue stopped"
             if not self._cmd_queue.empty():
@@ -438,7 +485,7 @@ class LiveExecutionEngine(ExecutionEngine):
         except asyncio.CancelledError:
             self._log.warning("Canceled task 'run_evt_queue'")
         except Exception as e:
-            self._log.error(f"{e!r}\n{traceback.format_exc()}")
+            self._log.exception(f"{e!r}", e)
         finally:
             stopped_msg = "Event message queue stopped"
             if not self._evt_queue.empty():
@@ -453,9 +500,7 @@ class LiveExecutionEngine(ExecutionEngine):
                 try:
                     await self._check_inflight_orders()
                 except Exception as e:
-                    self._log.error(
-                        f"Failed to check in-flight orders: {e!r}\n{traceback.format_exc()}",
-                    )
+                    self._log.exception("Failed to check in-flight orders", e)
         except asyncio.CancelledError:
             self._log.debug("Canceled task 'inflight_check_loop'")
 
@@ -470,12 +515,16 @@ class LiveExecutionEngine(ExecutionEngine):
         for order in inflight_orders:
             retries = self._inflight_check_retries[order.client_order_id]
             if retries >= self.inflight_check_max_retries:
+                self._inflight_check_retries.pop(order.client_order_id, None)
+                self._resolve_inflight_order(order)
                 continue
+
             ts_now = self._clock.timestamp_ns()
             ts_init_last = order.last_event.ts_event
             self._log.debug(
                 f"Checking in-flight order: {ts_now=}, {ts_init_last=}, {order=}...",
             )
+
             if ts_now > order.last_event.ts_event + self._inflight_check_threshold_ns:
                 self._log.debug(f"Querying {order} with exchange...")
                 query = QueryOrder(
@@ -490,6 +539,42 @@ class LiveExecutionEngine(ExecutionEngine):
                 self._execute_command(query)
                 self._inflight_check_retries[order.client_order_id] += 1
 
+    def _resolve_inflight_order(self, order: Order) -> None:
+        ts_now = self._clock.timestamp_ns()
+
+        if order.status == OrderStatus.SUBMITTED:
+            rejected = OrderRejected(
+                trader_id=order.trader_id,
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                account_id=order.account_id,
+                reason="UNKNOWN",
+                event_id=UUID4(),
+                ts_event=ts_now,
+                ts_init=ts_now,
+                reconciliation=True,
+            )
+            self._log.debug(f"Generated {rejected}")
+            self._handle_event(rejected)
+        elif order.status in (OrderStatus.PENDING_UPDATE, OrderStatus.PENDING_CANCEL):
+            canceled = OrderCanceled(
+                trader_id=order.trader_id,
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                venue_order_id=order.venue_order_id,
+                account_id=order.account_id,
+                event_id=UUID4(),
+                ts_event=ts_now,
+                ts_init=ts_now,
+                reconciliation=True,
+            )
+            self._log.debug(f"Generated {canceled}")
+            self._handle_event(canceled)
+        else:
+            raise RuntimeError(f"Invalid status for in-flight order, was '{order.status_string()}'")
+
     async def _own_books_audit_loop(self, interval_secs: float) -> None:
         try:
             while True:
@@ -498,7 +583,7 @@ class LiveExecutionEngine(ExecutionEngine):
         except asyncio.CancelledError:
             self._log.debug("Canceled task 'own_books_audit_loop'")
         except Exception as e:
-            self._log.error(f"Error auditing own books: {e}")
+            self._log.exception("Error auditing own books", e)
 
     async def _open_check_loop(self, interval_secs: float) -> None:
         try:
@@ -552,7 +637,52 @@ class LiveExecutionEngine(ExecutionEngine):
                     self._reconcile_order_report(report, trades=[])
         except Exception as e:
             # Catch all exception for error visibility in task
-            self._log.error(f"Error in check_open_orders: {e}")
+            self._log.exception("Error in check_open_orders", e)
+
+    async def _purge_closed_orders_loop(self, interval_mins: int) -> None:
+        interval_secs = interval_mins * 60
+        buffer_mins = self.purge_closed_orders_buffer_mins or 0
+        buffer_secs = buffer_mins * 60
+
+        try:
+            while True:
+                await asyncio.sleep(interval_secs)
+                ts_now = self._clock.timestamp_ns()
+                self._cache.purge_closed_orders(ts_now=ts_now, buffer_secs=buffer_secs)
+        except asyncio.CancelledError:
+            self._log.debug("Canceled task 'purge_closed_orders_loop'")
+        except Exception as e:
+            self._log.exception("Error purging closed orders", e)
+
+    async def _purge_closed_positions_loop(self, interval_mins: int) -> None:
+        interval_secs = interval_mins * 60
+        buffer_mins = self.purge_closed_positions_buffer_mins or 0
+        buffer_secs = buffer_mins * 60
+
+        try:
+            while True:
+                await asyncio.sleep(interval_secs)
+                ts_now = self._clock.timestamp_ns()
+                self._cache.purge_closed_positions(ts_now=ts_now, buffer_secs=buffer_secs)
+        except asyncio.CancelledError:
+            self._log.debug("Canceled task 'purge_closed_positions_loop'")
+        except Exception as e:
+            self._log.exception("Error purging closed positions", e)
+
+    async def _purge_account_events_loop(self, interval_mins: int) -> None:
+        interval_secs = interval_mins * 60
+        lookback_mins = self.purge_account_events_lookback_mins or 0
+        lookback_secs = lookback_mins * 60
+
+        try:
+            while True:
+                await asyncio.sleep(interval_secs)
+                ts_now = self._clock.timestamp_ns()
+                self._cache.purge_account_events(ts_now=ts_now, lookback_secs=lookback_secs)
+        except asyncio.CancelledError:
+            self._log.debug("Canceled task 'purge_account_events_loop'")
+        except Exception as e:
+            self._log.exception("Error purging account events", e)
 
     # -- RECONCILIATION -------------------------------------------------------------------------------
 
