@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -88,9 +88,8 @@ class TradingNode:
         # Async tasks
         self._task_streaming: asyncio.Future | None = None
 
-        # Operation flags
+        # State flags
         self._is_built = False
-        self._is_running = False
 
     @property
     def trader_id(self) -> TraderId:
@@ -164,7 +163,6 @@ class TradingNode:
         """
         return self.kernel.portfolio
 
-    @property
     def is_running(self) -> bool:
         """
         Return whether the trading node is running.
@@ -174,9 +172,8 @@ class TradingNode:
         bool
 
         """
-        return self._is_running
+        return self.kernel.is_running()
 
-    @property
     def is_built(self) -> bool:
         """
         Return whether the trading node clients are built.
@@ -306,7 +303,6 @@ class TradingNode:
                     "Run `node.build()` prior to start.",
                 )
 
-            self._is_running = True
             await self.kernel.start_async()
 
             if self.kernel.loop.is_running():
@@ -371,8 +367,6 @@ class TradingNode:
 
         await self.kernel.stop_async()
 
-        self._is_running = False
-
     def dispose(self) -> None:
         """
         Dispose of the trading node.
@@ -384,7 +378,7 @@ class TradingNode:
             timeout = self.kernel.clock.utc_now() + timedelta(
                 seconds=self._config.timeout_disconnection,
             )
-            while self._is_running:
+            while self.kernel.is_running():
                 time.sleep(0.1)
                 if self.kernel.clock.utc_now() >= timeout:
                     self.kernel.logger.warning(
@@ -413,30 +407,23 @@ class TradingNode:
                 self.kernel.logger.info("Shutting down executor")
                 self.kernel.executor.shutdown(wait=True, cancel_futures=True)
 
-            self.kernel.logger.info("Stopping event loop")
-            self.kernel.cancel_all_tasks()
-            self.kernel.loop.stop()
+            loop = self.kernel.loop
+
+            if not loop.is_closed():
+                if loop.is_running():
+                    self.kernel.logger.info("Stopping event loop")
+                    self.kernel.cancel_all_tasks()
+                    loop.stop()
+                else:
+                    self.kernel.logger.info("Closing event loop")
+                    loop.close()
+            else:
+                self.kernel.logger.info("Event loop already closed (normal with asyncio.run)")
         except (asyncio.CancelledError, RuntimeError) as e:
             self.kernel.logger.exception("Error on dispose", e)
         finally:
-            if self.kernel.loop.is_running():
-                self.kernel.logger.warning("Cannot close a running event loop")
-            else:
-                self.kernel.logger.info("Closing event loop")
-                self.kernel.loop.close()
-
-            # Check and log if event loop is running
-            if self.kernel.loop.is_running():
-                self.kernel.logger.warning(f"loop.is_running={self.kernel.loop.is_running()}")
-            else:
-                self.kernel.logger.info(f"loop.is_running={self.kernel.loop.is_running()}")
-
-            # Check and log if event loop is closed
-            if not self.kernel.loop.is_closed():
-                self.kernel.logger.warning(f"loop.is_closed={self.kernel.loop.is_closed()}")
-            else:
-                self.kernel.logger.info(f"loop.is_closed={self.kernel.loop.is_closed()}")
-
+            self.kernel.logger.info(f"loop.is_running={self.kernel.loop.is_running()}")
+            self.kernel.logger.info(f"loop.is_closed={self.kernel.loop.is_closed()}")
             self.kernel.logger.info("DISPOSED")
 
     def _loop_sig_handler(self, sig: signal.Signals) -> None:
