@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------------------------
-#  Copyright (C) 2015-2024 Nautech Systems Pty Ltd. All rights reserved.
+#  Copyright (C) 2015-2025 Nautech Systems Pty Ltd. All rights reserved.
 #  https://nautechsystems.io
 #
 #  Licensed under the GNU Lesser General Public License Version 3.0 (the "License");
@@ -24,14 +24,15 @@ from nautilus_trader.model.instruments.binary_option import BinaryOption
 from nautilus_trader.model.instruments.cfd import Cfd
 from nautilus_trader.model.instruments.commodity import Commodity
 from nautilus_trader.model.instruments.crypto_future import CryptoFuture
+from nautilus_trader.model.instruments.crypto_option import CryptoOption
 from nautilus_trader.model.instruments.crypto_perpetual import CryptoPerpetual
 from nautilus_trader.model.instruments.currency_pair import CurrencyPair
 from nautilus_trader.model.instruments.equity import Equity
 from nautilus_trader.model.instruments.futures_contract import FuturesContract
 from nautilus_trader.model.instruments.futures_spread import FuturesSpread
 from nautilus_trader.model.instruments.index import IndexInstrument
-from nautilus_trader.model.instruments.options_contract import OptionsContract
-from nautilus_trader.model.instruments.options_spread import OptionsSpread
+from nautilus_trader.model.instruments.option_contract import OptionContract
+from nautilus_trader.model.instruments.option_spread import OptionSpread
 from nautilus_trader.model.instruments.synthetic import SyntheticInstrument
 
 
@@ -41,6 +42,7 @@ __all__ = [
     "Cfd",
     "Commodity",
     "CryptoFuture",
+    "CryptoOption",
     "CryptoPerpetual",
     "CurrencyPair",
     "Equity",
@@ -48,8 +50,8 @@ __all__ = [
     "FuturesSpread",
     "IndexInstrument",
     "Instrument",
-    "OptionsContract",
-    "OptionsSpread",
+    "OptionContract",
+    "OptionSpread",
     "SyntheticInstrument",
     "instruments_from_pyo3",
 ]
