@@ -382,14 +382,21 @@ class ParquetDataCatalog(BaseDataCatalog):
         fs.mkdirs(path, exist_ok=True)
         name = basename_template.format(i=0)
         parquet_file = f"{path}/{name}.parquet"
+        empty_file = parquet_file
+        i = 0
 
-        if mode == CatalogWriteMode.NEWFILE:
-            i = 0
+        while Path(empty_file).exists():
+            i += 1
+            name = basename_template.format(i=i)
+            empty_file = f"{path}/{name}.parquet"
 
-            while Path(parquet_file).exists():
-                i += 1
-                name = basename_template.format(i=i)
-                parquet_file = f"{path}/{name}.parquet"
+        if i > 1 and mode != CatalogWriteMode.NEWFILE:
+            print(
+                "Warning, Only CatalogWriteMode::NEWFILE is allowed for a directory containing several parquet files. Aborting write_data.",
+            )
+            return
+        elif mode == CatalogWriteMode.NEWFILE:
+            parquet_file = empty_file
 
         # following solution from https://stackoverflow.com/a/70817689
         if (
@@ -563,9 +570,9 @@ class ParquetDataCatalog(BaseDataCatalog):
 
         if not is_nautilus_class(data_cls):
             # Special handling for generic data
+            metadata = kwargs.get("metadata")
             data = [
-                CustomData(data_type=DataType(data_cls, metadata=kwargs.get("metadata")), data=d)
-                for d in data
+                CustomData(data_type=DataType(data_cls, metadata=metadata), data=d) for d in data
             ]
 
         return data
@@ -818,8 +825,10 @@ class ParquetDataCatalog(BaseDataCatalog):
 
         if start is not None:
             filters.append(pds.field(ts_column) >= pd.Timestamp(start).value)
+
         if end is not None:
             filters.append(pds.field(ts_column) <= pd.Timestamp(end).value)
+
         if filters:
             filter_ = combine_filters(*filters)
         else:
@@ -929,18 +938,6 @@ class ParquetDataCatalog(BaseDataCatalog):
 
     # -- OVERLOADED BASE METHODS ------------------------------------------------------------------
 
-    def instruments(
-        self,
-        instrument_type: type | None = None,
-        instrument_ids: list[str] | None = None,
-        **kwargs: Any,
-    ) -> list[Instrument]:
-        return super().instruments(
-            instrument_type=instrument_type,
-            instrument_ids=instrument_ids,
-            **kwargs,
-        )
-
     def _list_directory_stems(self, subdirectory: str) -> list[str]:
         glob_path = f"{self.path}/{subdirectory}/*"
         return [Path(p).stem for p in self.fs.glob(glob_path)]
